#
# Chum::Analysis::Local::CoexpressionScanner
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::CoexpressionScanner - 

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses TFBS::Perl modules (Jaspar).  MatrixDB should be jaspar

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::CoexpressionScanner;

use vars qw(@ISA @CS_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Chum::Analysis::ApplicationI;
use Chum::Analysis::Local::MatrixScanner;
use Data::Dumper;
use File::Temp qw/ tempfile tempdir unlink0 /;

@ISA = qw( Chum::Analysis::ApplicationI );
BEGIN {
        @CS_PARAMS = qw(coexpression_file opossum_install_dir matrix_db);
        foreach my $attr (@CS_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	if (!defined($self->{'matrix_db'})) {
		$self->throw("matrix_db not defined");
	} elsif (!$self->{'matrix_db'}->isa("Chum::DB::MatrixDB")) {
		$self->throw("matrix_db is not a Chum::DB::MatrixDB object");	
	} 
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;
        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

###################################################################################
########## RUN THE VARIATION ANALYSIS FOR TRANSFAC
###################################################################################

sub run_variation_analysis {
	my $self = shift;
	my $stack_object = shift;
	my $ensembl_gene_name = shift;
	if (!defined($ensembl_gene_name)) { $self->throw("No ensembl gene id defined\n"); }

	my @coexpressed_genes = @{$self->_get_coexpressed_genes($ensembl_gene_name, 3)};
	print Dumper \@coexpressed_genes;
	if (@coexpressed_genes) {
		my ($factors_arrayref, $excluded_genes_arrayref) = $self->_get_factors_for_coexpressed_genes(\@coexpressed_genes);
		my $matrixset = $self->{'matrix_db'}->_find_pwm_matrixset($factors_arrayref); ##have a set of factors, reapply to stack, ref and var.
		$self->{'matrix_db'}->matrixset($matrixset);
		my $scanner = new Chum::Analysis::Local::MatrixScanner('matrix_db' => $self->{'matrix_db'});  ##Defining our own matrix set and running analysis
		return $scanner->run_variation_analysis($stack_object);
	} else {
		warn "Could not find any coexpressed genes for $ensembl_gene_name";
		return undef;
	}
}

sub _get_factors_for_coexpressed_genes {
	my $self = shift;
	my $coexpressed_genes_arrayref = shift;
	my @coexpressed_genes = @$coexpressed_genes_arrayref;
	if (@coexpressed_genes) {
		my ($fh, $filename) = tempfile( "coexpressiontmpXXXX", SUFFIX => '.genes');
		foreach my $cg (@coexpressed_genes) {
			print $fh $cg . "\n";
		}
		my ($ofh, $outfilename) = tempfile( "coexpressiontmpXXXX", SUFFIX => '.out');
		my ($efh, $exclusionfilename) = tempfile( "coexpressiontmpXXXX", SUFFIX => '.excluded');
		my $command = "perl58 " . $self->{'opossum_install_dir'} . "/scripts/default_analysis.pl -g " . $filename . " -f " . $outfilename . " 2>&1 > " . $exclusionfilename;
		system($command);
		unlink0($fh, $filename);
		open(OUT, $outfilename) || $self->throw("Cannot open oPOSSUM result file");
		my @factors = ();
		my %scores;
		while (my $line = <OUT>) {
			chomp($line);
			if ($line =~ /([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]*/g) {
				my $factor = $1;
				my $fscore = $6;
				if ($fscore > 0) {
					$scores{$factor} = $fscore;
					push @factors, $factor;
					#print STDERR "Using $factor\n";
				}
			}
		}
		close(OUT);
		unlink0($ofh, $outfilename);
		open(EXC, $exclusionfilename) || $self->("Cannot open exclusion file for oPOSSUM");
		my @excluded_genes = ();
		while (my $line = <EXC>) {
			chomp($line);
			if ($line =~ /Gene[\s]*([\S]+)[\s]*not[\S\s]+excluding[\S\s]+/gi) {
				push @excluded_genes, $1;
			}
		}
		close(EXC);
		unlink0($efh, $exclusionfilename);
		return \@factors, \%scores, \@excluded_genes;
	}
	return undef;
}


##Needs coexpression data file in format ENSGXXXX	ENSGXXXX	1
sub _get_coexpressed_genes {
	my $self = shift;
	my $ensembl_gene_name = shift;
	my $min_score = shift;

	open(COEXPRESSION_DATA, $self->{'coexpression_file'}) || $self->throw("No coexpression data file found");
	my @coexpressed_genes = ();
	while (my $record = <COEXPRESSION_DATA>) {
		chomp($record);
		if ($record =~ /(ENSG[0-9]+)[\s]+(ENSG[0-9]+)[\s]+([\S]+)/g) {
			my $gene1 = $1;
			my $gene2 = $2;
			my $score = $3;

			if ($gene1 eq $ensembl_gene_name) {
				push @coexpressed_genes, {'gene' => $gene2, 'score' => $score};
			} elsif ($gene2 eq $ensembl_gene_name) {
				push @coexpressed_genes, {'gene' => $gene1, 'score' => $score};
			}
		}
	}
	close(COEXPRESSION_DATA);
	my %genehash;
	foreach my $gene (@coexpressed_genes) {
		if (!defined($genehash{$gene->{'gene'}})) {
			$genehash{$gene->{'gene'}} = abs($gene->{'score'});
		} else {
			$genehash{$gene->{'gene'}} += abs($gene->{'score'});
		}
	}

	my @ret_genes = (); 
	for my $key (keys %genehash) {
		if ($genehash{$key} >= $min_score) {
			push @ret_genes, $key;
		}
	}

	return \@ret_genes;
}
