#
# Chum::Analysis::Local::Motifsampler
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::Motifsampler - Runs local version of motifsampler

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::Motifsampler;

use vars qw(@ISA @MOTIFSAMPLER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Seq;
use Bio::SeqIO;
use Chum::Analysis::Local::LocalApplicationI;
use File::Copy;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw( Chum::Analysis::Local::LocalApplicationI );
BEGIN {
        @MOTIFSAMPLER_PARAMS = qw();
        foreach my $attr (@MOTIFSAMPLER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	$self->program("motifsampler");
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub run_program {
	my $self = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $results_file = shift;
	my $meta_input_hashref = shift;
	my $named_supplementary_result_files_hashref = shift;

	if (!defined($meta_input_hashref->{'background_file'})) {
		$self->throw("No background file defined\n");
	}
	my ($tmpfh, $tmpfilename) = tempfile( "motifsamplerXXXXXX", SUFFIX => '.fa', DIR => "/tmp/", UNLINK => 1);
	my ($tmpmatrixfh, $tmpmatrixfilename) = tempfile( "motifsamplerXXXXXX", SUFFIX => '.matrix', DIR => "/tmp/", UNLINK => 1);
	$named_supplementary_result_files_hashref->{'matrix_file'} = $tmpmatrixfilename;
	
	my $seqio = new Bio::SeqIO( -file => ">$tmpfilename",
				    -format => "Fasta");
	$seqio->write_seq(new Bio::Seq(-seq => $reference_sequence->sequence->seq,
				       -id => $reference_sequence->id));
	foreach my $orthologous_sequence (@{$stack_object->orthologous_sequences}) {
		$seqio->write_seq(new Bio::Seq(	-id => $orthologous_sequence->id,
						-seq => $orthologous_sequence->seq));
	}
	
	my $commandline = $self->binary . " -f $tmpfilename";
	$commandline .= " -b " . $meta_input_hashref->{'background_file'};
	foreach my $key (keys %$meta_input_hashref) {
		if ($key =~ /-([\S]+)/g) {
			if ($1 ne "m") {
				$commandline .= " " . $key . " "  . $meta_input_hashref->{$key};
			} else {
				warn "Matrix file with -m parameter is automatically set\n";	
			}
		}
	}
	$commandline .= " -o $results_file -m $tmpmatrixfilename";
	print STDERR "COMMAND: $commandline\n";
	system($commandline);
	unlink0($tmpfh, $tmpfilename);
	if ($? == -1) {
		$self->throw("ERROR: Failed to execute: $!");
	}
	return 0;
}

sub convert_output_to_xml {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;
	my $matrix_file = undef;
	my $cmd = undef;
	if(@_) {
		$matrix_file = shift;
	}
	if (defined($matrix_file)) {
		$cmd = $ENV{'PERLEXE'} . " " . $ENV{'CHUM_INSTALL_DIR'} . "/scripts/motifsamplerToXML.pl -f " . $input_file . " -o $output_file -m $matrix_file";	
	} else {
		$cmd = $ENV{'PERLEXE'} . " " . $ENV{'CHUM_INSTALL_DIR'} . "/scripts/motifsamplerToXML.pl -f " . $input_file . " -o $output_file";
	}
	
	print STDERR "Converting MotifSampler output to XML: " . $cmd . "\n";
	system($cmd);
	if ($? == -1) {
		$self->throw("ERROR: Failed to execute: $!");
	}
}

1;