#
# Chum::Analysis::Remote::Motifsampler
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Remote::Motifsampler - Runs motifsampler background creator

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses the Chinook platform (Montgomery et al. Nature Methods, August 2005) to remotely
run Motifsampler against a set of input sequences.  Results are returned as
GFF from the start of the reference input sequence.

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Remote::Motifsampler;

use vars qw(@ISA @MOTIFSAMPLER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Chum::Analysis::Remote::RemoteApplicationI;

@ISA = qw( Chum::Analysis::Remote::RemoteApplicationI );
BEGIN {
        @MOTIFSAMPLER_PARAMS = qw();
        foreach my $attr (@MOTIFSAMPLER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub program {
	my $self = shift;
	return "MotifSampler-CreateBackgroundModel";

}

sub get_meta_input_hashref {
	my $self = shift;
	my %meta_hash;
	$meta_hash{'fasta_sequence_file'} = '';
	return \%meta_hash;
}

sub load_data_entry_object_set {
	my $self = shift;
	my $deosets_arrayref = shift;
	my $service_object = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $meta_input_hashref = shift;

	if (!defined($meta_input_hashref{'fasta_sequence_file'})) { $self->throw('ERROR: MotifSamplerBackground requires a fasta sequence file as input');}
	unless (-e $meta_input_hashref{'fasta_sequence_file'}) { $self->throw('ERROR: MotifSamplerBackground requires a fasta sequence file as input');}

	my $loaded = 0;
	my @deosets = @{$batch->getDEOSets()};
	foreach my $deoset (@deosets) {
		my @deos;
		my @required_deo_names = @{$deoset->getAllowedDEOTypes()};
		foreach my $required_deo_name (@required_deo_names) {
			if ($required_deo_name eq "DNA_FILE") {
				$loaded++;
				my $deo = $self->chinook_manager->getDEO($required_deo_name);
				my $file_id = $self->chinook_manager->uploadFile($service_object, $meta_input_hashref->{'fasta_sequence_file'});
				if (!defined($file_id)) {
					unlink($tmpstackfile);
					$self->throw( "ERROR: Failed, could not upload file to server\n");
				} else {
					unlink($tmpstackfile);
				}
				$deo->setName("SEQUENCE"); ##A unique name for the object
				$deo->setOrder("-1");   ##The precedence of the object in the set
				my $attributes = $deo->getAttributes(); ##The collection of attributes
				while ( ( my $key, my $value ) = each(%$attributes) ) {
					if ($key eq "getMode") { %$attributes->{$key}->{0} = $service_object->getMode(); }
					elsif ($key eq "getFilewireUri") { %$attributes->{$key}->{0} = $service_object->getFilewireUri(); }
					elsif ($key eq "getAbsoluteFileName") { %$attributes->{$key}->{0} = $tmpstackfile;  }
					elsif ($key eq "getFileType") { %$attributes->{$key}->{0} = "fasta - dna"; }
					elsif ($key eq "getFileId") { %$attributes->{$key}->{0} = $file_id; }
					elsif ($key eq "getName") { #do nothing 
					}
					else {
						$self->warn("Unknown key $key");
					}
				}
				push @deos, $deo;
			}
		}
		$deoset->setDEOs(\@deos);
	}
	return $loaded;
}

1;