#
# Chum::Analysis::Remote::RemoteApplicationI
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Remote::RemoteApplicationI - A remote application interface

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses the Chinook platform (Montgomery et al. Nature Methods, August 2005) to remotely
run an application, Chinook needs to be installed and running

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Remote::RemoteApplicationI;

use vars qw(@ISA @REMOTE_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Tools::Run::Chinook::Batch;
use Bio::Tools::Run::Chinook::ChinookManager;
use Bio::Tools::Run::Chinook::QueueBatch;
use Bio::Tools::Run::Chinook::Report;
use Bio::Tools::GFF;
use Cwd;
use Data::Dumper;
use Chum::Analysis::ApplicationI;
use Chum::Result::Result;
use Chum::Result::ResultSet;
use File::Temp;

@ISA = qw( Chum::Analysis::ApplicationI );
BEGIN {
        @REMOTE_PARAMS = qw(chinook_manager);
        foreach my $attr (@REMOTE_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		$self->{'chinook_manager'} = Bio::Tools::Run::Chinook::ChinookManager->new( machine_name => "localhost", port => "7999"); 
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub chinook_manager {
	my $self = shift;
	return $self->{'chinook_manager'};
}


sub get_input_parameters {
	my $self = shift;
	my $service_object = shift;
	if (!defined($service_object)) {
		$self->throw("No service defined, cannot get remote parameters");
	}
	my $batch_object = Bio::Tools::Run::Chinook::Batch->new ( batch_filename => $service_object->getBatchFilename() );
	return $batch_object->getParameters();
}

sub get_meta_input_hashref {
	shift->throw_not_implemented();
}

sub get_remote_services {
	my $self = shift;
    my @services = @{$self->chinook_manager->getServices()};
	my @ret_services;
	foreach my $service (@services) {
        if ($service->getName eq $self->program) {
			push @ret_services, $service;
		}
	}
	return \@ret_services;
}

###################################################################################
#  USE CHINOOK TO RUN SERVICE
###################################################################################

sub load_data_entry_object_set {
	shift->throw_not_implemented();
}

sub run_service {
	my $self = shift;
    my $service_object = shift;
	my $reference_sequence = shift;
    my $stack_object = shift;
    my $parameters_ref = shift;
	my $outputfile = shift;
	my $meta_input_hash = shift;
	
	my $batch_queue_path = $self->chinook_manager->getBatchQueueDirectoryPath();
    my $batch = Bio::Tools::Run::Chinook::Batch->new ( batch_filename => $service_object->getBatchFilename() );
    
	##1. Load data into Chinook
	my @deosets = @{$batch->getDEOSets()};
	unless ($self->load_data_entry_object_set(\@deosets, $service_object, $reference_sequence, $stack_object, $meta_input_hash)) {
		$self->throw("Could not load DEOSets for " . $self->program);
		return 0;
	}


	##2. Run application over chinook
	my $rand = int(rand(1000000));
	my $batch_queue_file = $batch_queue_path . "chinook." . $self->program . "." . $rand . ".batchqueue"; ##The input instruction file to Chinook
	my $queue_batch = Bio::Tools::Run::Chinook::QueueBatch->new(	deosets => \@deosets,
                                                                    parameters => $parameters_ref,
																	batch => $batch,
																	filename => $batch_queue_file);
	my $id = $queue_batch->writeQueueBatch();
	my $submitted = $self->chinook_manager->processQueueBatch($queue_batch->getFilename);

	##3. Check that Chinook job has completed.
	if ($submitted) {

				##3a. Wait until report is ready
				my $tmp_report_file = getcwd . "/report." . $self->program . "." . $rand . ".tmp"; ##The report file 
                my $report_filename;
                my $report_not_done = 1;  
                while ($report_not_done) {  
                        $report_filename = $self->chinook_manager->isReportReady($id, $tmp_report_file);
                        if (defined($report_filename)) {
                                $report_not_done = 0; #false
                        }
                }

                ##3b. Get the canonical results file
                my $report = Bio::Tools::Run::Chinook::Report->new( report_filename => $report_filename);
                if ($report->getStatus() eq "FAILED") {
						print "Job failed execution " . $report->getFailureMessage() . "\n";
                        $self->throw("Job failed execution " . $report->getFailureMessage());
                } elsif ($report->getStatus() eq "SUCCESS") {
                        my $report_file = $report->getCanonicalReportFile();
                        $self->chinook_manager->downloadFile($service_object, $report_file->getFileId(), $outputfile);
                        unlink($tmp_report_file);
						unlink($batch_queue_file);
                        return 0; ##Success
                } else {
                        $self->throw("Unknown job status: " . $report->getStatus());
                }
                unlink($tmp_report_file);
				unlink($batch_queue_file); 
        }
        return 1;
}

sub _convert_output_to_gff {
	shift->throw_not_implemented();
}

###################################################################################
########## RUN VARIATION ANALYSIS
###################################################################################

sub run_variation_analysis {
        my $self = shift;
        my $service_object = shift;
        my $stack_object = shift;
        my $parameters_arrayref = shift;
		my $meta_input_hash = shift;

        if (!defined($service_object)) { $self->warn("No service defined"); return undef; }
        elsif (!defined($stack_object)) { $self->warn("No stack defined"); return undef; }
        elsif (!defined($stack_object->reference_sequence)) { $self->warn("No reference sequence defined"); return undef; }
		elsif (scalar(@{$stack_object->orthologous_sequences}) == 0) { $self->warn("No orthologs"); return undef; }

        my @results;

        ####RUN THE CANONICAL ANALYSIS
        my $tmp_file = "/tmp/" . $self->program . ".run." . rand(10000000) . ".tmp";
		my $tmp_gff_file = "/tmp/" . $self->program . ".run." . rand(10000000) . ".gff";
        my $status = $self->run_service(  	$service_object,
                                            $stack_object->reference_sequence,
											$stack_object,
											$parameters_arrayref,
											$tmp_file,
											$meta_input_hash);

        if ($status) { $self->warn("Could not obtain " . $self->program . " results"); return undef; }
		$self->_convert_output_to_gff($tmp_file, $tmp_gff_file);

        ####LOAD IN THE CANONICAL ANALYSIS
        my $gffio = Bio::Tools::GFF->new(-file => $tmp_gff_file, -gff_version => 2);
        my $feature;
        my @canonical_features;
        while ($feature = $gffio->next_feature()) {
                push @canonical_features, $feature;
        }
        push @results, new Chum::Result::Result(    'type' => 'CANONICAL',
													'seq' => $stack_object->reference_sequence,
													'seqfeat' => \@canonical_features);
	
        ####RUN THE VARIATION ANALYSES
        foreach my $variant_sequence (@{$stack_object->variant_sequences}) {
                $status = $self->run_service(	$service_object,
                                                $variant_sequence,
                                                $stack_object,
                                                $parameters_arrayref,
                                                $tmp_file,
												$meta_input_hash);

                if ($status) { $self->warn("Could not obtain " . $self->program . " results: variant analysis"); return undef; }
				$self->_convert_output_to_gff($tmp_file, $tmp_gff_file);
                my $gffio = Bio::Tools::GFF->new(-file => $tmp_gff_file, -gff_version => 2);
                my $feature;
                my @variant_features;
                while ($feature = $gffio->next_feature()) {
                        push @variant_features, $feature;
                }
                push @results, new Chum::Result::Result(	'type' => 'VARIANT',
															'seq' => $variant_sequence,
															'seqfeat' => \@variant_features);
        }
		unlink($tmp_file);
		unlink($tmp_gff_file);

        ####PUT TOGETHER THE RESULTSET FOR RETURN ANALYSIS
        my $resultset = new Chum::Result::ResultSet(	'stack' => $stack_object,
														'program' => $self->program,
                                                        'results' => \@results);

        return $resultset;
}


1;