#
# Chum::Background
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Background - Background Sequence Object

=head1 SYNOPSIS
 
 my @genes = ['gene1', 'gene2', 'gene3']; #A list of gene names, for reproducibility
 my @stacks = [$genestack1, $genestack2, $genestack3]; #Stack objects for each gene (typically contains orthologous sequences)
 my $backgroundseq = "ATATATATATATATATATATAT"; #A generated background sequence from the the Stack objects (generated from the BackgroundBuilder module).
 my $background = new Background('genes' => \@genes,
				 'stacks' => \@stacks,
				 'backgroundseq' => $backgroundseq);

=head1 DESCRIPTION

A background sequence generated from stacks associated to genes.
For more information, read the definition of the BackgroundBuilder object.

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Background;

use vars qw(@ISA @BACKGROUND_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @BACKGROUND_PARAMS = qw(genes stacks backgroundseq);
        foreach my $attr (@BACKGROUND_PARAMS)
                { $OK_FIELD{$attr}++; }
}

=head2 new

 Usage   : my $background = new Background(	'genes' => \@genes,
						'stacks' => \@stacks,
						'backgroundseq' => $backgroundseq);
		
 Returns : Chum::Background
 Args    : an arrayref to a gene list, an arrayref to a Chum::Stack object list, and a background sequence as a string 

=cut

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

=head2 genes

 Title	 : genes
 Usage   : my @genes = @{$background->genes()};
 Function: The names of the genes, scanned to make this background (for reproducibility)
 Returns : an arrayref of gene ids
 Args    : none

=cut

sub genes { 
	my $self = shift;
	return $self->{'genes'};
}

=head2 stacks

 Title	 : stacks
 Usage   : my @stacks = @{$background->stacks()};
 Function: The gene stacks, scanned to make this background (for reproducibility)
 Returns : an arrayref of Chum::Stack objects
 Args    : none

=cut

sub stacks {
	my $self = shift;
        return $self->{'stacks'};
}

=head2 backgroundseq

 Title	 : backgroundseq
 Usage   : my $backgroundseq = $background->backgroundseq();
 Function: The generated background sequence
 Returns : a nucleotide string
 Args    : none

=cut

sub backgroundseq {
	my $self = shift;
        return $self->{'backgroundseq'};
}

=head2 save_background_to_file

 Title	 : save_background_to_file
 Usage   : $background->save_background_to_file($filename);
 Function: Saves the background object to a file (serialization)
 Returns : 
 Args    : none

=cut

sub save_background_to_file {
	my $self = shift;
	my $filename = shift;

	open(IFILE, ">$filename") || $self->throw("Cannot open $filename\n");
	my @stacks = @{$self->{'stacks'}};
	foreach my $stack (@stacks) { print IFILE "S" . $stack . "\n"; }
	my @genes = @{$self->{'genes'}};
	foreach my $gene (@genes) { print IFILE "G" . $gene . "\n"; }
	print "B" . $self->{'backgroundseq'} . "\n";
	close(IFILE);
}

=head2 load_background_from_file
 
 Title	 : load_background_from_file
 Usage   : my $background = $background->load_background_from_file($filename);
 Function: Loads the background object from a file (serialization)
 Returns : 
 Args    : none
 Throws	 : Exception if file cannot be opened or if file contents do not match serialization format

=cut

sub load_background_from_file {
	my $self = shift;
	my $filename = shift;
	
	my @stacks;
        my @genes;
	my $backgroundseq;

	open(IFILE, $filename) || $self->throw("Cannot open $filename\n");
	foreach my $line (<IFILE>) {
		chomp($line);
		if ($line =~ /S([\S\s]+)/g) {
			push @stacks, $1;
		} elsif ($line =~ /G([\S\s]+)/g) {
			push @genes, $1;
		} elsif ($line =~ /B([\S\s]+)/g) {
			$backgroundseq = $1;
		} else {		
			$self->throw("Undefined line: $line");
		}
	}
	close(IFILE);	

	$self->{'genes'} = \@genes;
	$self->{'stacks'} = \@stacks;
	$self->{'backgroundseq'} = $backgroundseq;
}

1;
