#
# Chum::Backgroundbuilder
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Backgroundbuilder - Builds Background objects

=head1 SYNOPSIS

=head1 DESCRIPTION

Builds a background sequence from randomly selected non-overlapping genes on the same chromosome as
an input target gene.

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Backgroundbuilder;

use vars qw(@ISA @BACKGROUNDBUILDER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;
use Chum::Background;
use Chum::Stackbuilder;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @BACKGROUNDBUILDER_PARAMS = qw(ensgid minmaf maxmaf rangecheck);
        foreach my $attr (@BACKGROUNDBUILDER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

my $BACKGROUND_SEARCH_SPACE = 1000000;
my $GENE_COUNT = 2;

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }

        if (!defined($self->{'ensgid'})) {
                $self->throw( "Required parameter: ensgid not defined");
        }
        if (!defined($self->{'minmaf'})) { $self->{'minmaf'} = 0; } ##min maf greater than 0 as default
        if (!defined($self->{'maxmaf'})) { $self->{'maxmaf'} = 0.5; }  ##max maf less than 0.5 as default
        if ($self->{'maxmaf'} < $self->{'minmaf'}) {
                $self->throw(   "The min minor allele frequency " .
                                "must be less than the max minor allele frequency\n");
        }

        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub generate_background {
	my $self = shift;
	my $generate_di_paths = shift;
	if (!defined($generate_di_paths)) {
		$generate_di_paths = 0; ##FALSE
	}

	##CONNECT TO ENSEMBL####################################################
    my $reg = "Bio::EnsEMBL::Registry";
    $reg->load_all();

	##GET THE TARGET GENE###################################################
    my $ga = $reg->get_adaptor("human", "core", "Gene");
    my $gene = $ga->fetch_by_stable_id($self->{'ensgid'});

    if (!defined($gene)) {
		$self->throw("Could not find EnsEMBL gene\n");
	}
	$gene = $gene->transform('toplevel');
	my $sa = $reg->get_adaptor("human", "core", "Slice");
	my $slice = $sa->fetch_by_region('chromosome', $gene->slice->seq_region_name, $gene->start, $gene->end, $gene->strand());

	##GET THE SEARCH GENES##################################################
	my $ref_slice = $slice;
	my $five_slice = $ref_slice->expand($BACKGROUND_SEARCH_SPACE, - ($ref_slice->length + 1));
	my $three_slice = $ref_slice->expand(-($ref_slice->length + 1), $BACKGROUND_SEARCH_SPACE);

	my @rgenes = @{$self->_get_random_genes($five_slice)};
	push @rgenes, @{$self->_get_random_genes($three_slice)};

	#DEBUG TEST
	for (my $index = 0; $index < scalar(@rgenes); $index++) {
		if (@rgenes[$index]->stable_id eq $self->{'ensgid'}) {
			print STDERR "WARNING: TARGET GENE INCLUDED IN BACKGROUND: WARNING!!!!!\n";
		}
	}
	
	##BUILD ALL THE STACKS AND THEN REMOVE REDUNDANT STACKS#################
	my %stacks;
	foreach my $rgene (@rgenes) {	
		my $rstackbuilder = new Chum::Stackbuilder( 	'ensgid' => $rgene->stable_id,
														'minmaf' => $self->{'minmaf'},
														'maxmaf' => $self->{'maxmaf'},
														'_memconserve' => 1);
		print STDERR "Building background from " . $rgene->stable_id . "...\n";
		my @rstacks = @{$rstackbuilder->generate_stacks_from_ensgid($rgene->stable_id)};
		foreach my $rstack (@rstacks) {
			%stacks->{$rstack->{'variation'}} = $rstack; ##remove duplicates in hash
		}
	}		

	print STDERR "Building background...\n";

	##GENERATE THE BACKGROUND FROM THE STACK HASH
	my $background_obj = $self->_build_background_obj(\@rgenes, \%stacks, $generate_di_paths);
	
	print STDERR "Background built!\n";

	return $background_obj;
}

sub _get_random_genes {
	my $self = shift;
	my $slice = shift;

	my @ret_genes;
	my @genes = @{$slice->get_all_Genes()};
	my $index = 0;
	while ($index < $GENE_COUNT) {
		my $length = scalar(@genes);
		if ($length > 0) {
			my $gindex = int(rand($length));
			my $gene = splice @genes, $gindex;
			push @ret_genes, $gene;
			$index++;
		} else {
			last;
		}
	}

	return \@ret_genes;
}

sub _build_background_obj {
	my $self = shift;
	my $rgenes_ref = shift;
	my $stack_ref = shift;
	my $generate_di_paths = shift;
	my %stacks = %$stack_ref;

	my @gene_names;
	foreach my $rgene (@$rgenes_ref) {
		push @gene_names, $rgene->stable_id;
	}
	
	my @seq_stack;
	for my $variation ( keys %stacks ) {
		my $stack = $stacks{$variation};
        my @seqs = @{$stack->stack};
		foreach my $seq (@seqs) {
			push @seq_stack, $seq;
		}
    }
	
	my $backgroundseq;
	if ($generate_di_paths) {
		print STDERR "Building background sequences...\n";
		$backgroundseq = $self->_build_background_sequence(\@seq_stack);
		print STDERR "Built background sequences...\n";
	}
	
	my $background = new Chum::Background( 	'genes' => \@gene_names,
												'stacks' => \@seq_stack,
												'backgroundseq' => $backgroundseq );

	

	return $background;
}

sub rebuild_background {
	my $self = shift;
	my $background = shift;

	my $backgroundseq = $self->_build_background_sequence($background->stacks);
	$background->backgroundseq = $backgroundseq;
	
	return $background;
}

sub _build_background_sequence {
	my $self = shift;
	my $seq_stack_ref = shift;
	
	my @a_dnts;
	my @t_dnts;
	my @c_dnts;
	my @g_dnts;
	
	foreach my $locseq (@$seq_stack_ref) {
		my $seq = $locseq->seq;
		for (my $i = 0; $i < length($seq); $i++) {
			my $dnt = uc(substr ($seq, $i, 2));
			if (length($dnt) == 2) {
				my $ldnt = substr($dnt, 0, 1);
				if ($ldnt eq 'A') { push @a_dnts, $dnt;
				} elsif ($ldnt eq 'C') { push @c_dnts, $dnt;
				} elsif ($ldnt eq 'G') { push @g_dnts, $dnt;
				} elsif ($ldnt eq 'T') { push @t_dnts, $dnt;
				} else { $self->throw("Unidentified dnt $ldnt from dinucleotide $dnt in " . $seq);
				}
			}
		}
	}
	
	my $backgroundseq;
	my $length = scalar(@a_dnts) + scalar(@c_dnts) + scalar(@g_dnts) + scalar(@t_dnts);	
	
	print STDERR "Building background dnt paths for " . $length . " sequence...\n";

	for (my $index = 0; $index < $length; $index++) {
		$backgroundseq .= $self->_get_dinucleotide($backgroundseq, \@a_dnts, \@c_dnts, \@g_dnts, \@t_dnts);
	}	

	return $backgroundseq;
}

##Find a path through
sub _get_dinucleotide {
	my $self = shift;
	my $background_seq = shift;
	my $a_dnts_ref = shift;
	my $c_dnts_ref = shift;
	my $g_dnts_ref = shift;
	my $t_dnts_ref = shift;

	my @a_dnts = @$a_dnts_ref;
	my @c_dnts = @$c_dnts_ref;
	my @g_dnts = @$g_dnts_ref;
	my @t_dnts = @$t_dnts_ref;

	my $a_length = scalar(@a_dnts);
    my $c_length = scalar(@c_dnts);
    my $g_length = scalar(@g_dnts);
    my $t_length = scalar(@t_dnts);

	my $dnt;
	my $array_selector;
	my $lastletter;
        if (defined ($background_seq)) {
			$lastletter = substr($background_seq, length($background_seq) - 1);

			if ($lastletter eq 'A') {
				my $index = int (rand($a_length));
				$dnt = splice(@a_dnts, $index, 1);
				$dnt = substr($dnt, 1);
			} elsif ($lastletter eq 'C') {
				my $index = int (rand($c_length));
				$dnt = splice(@c_dnts, $index, 1);
				$dnt = substr($dnt, 1);
			} elsif ($lastletter eq 'G') {
				my $index = int (rand($g_length));
				$dnt = splice(@g_dnts, $index, 1);
				$dnt = substr($dnt, 1);
			} elsif ($lastletter eq 'T') {
				my $index = int (rand($t_length));
				$dnt = splice(@t_dnts, $index, 1);
				$dnt = substr($dnt, 1);
			} else {
				$self->throw("Undefined array selector value $array_selector");
			}
        } else {
		my $r_max = 0;
		my @arrays;
		
		if ($a_length > 0) { $r_max++; push @arrays, "A"; }
		if ($c_length > 0) { $r_max++; push @arrays, "C";}
		if ($g_length > 0) { $r_max++; push @arrays, "G";}
		if ($t_length > 0) { $r_max++; push @arrays, "T";}

		$array_selector = int ( rand ($r_max));
		my $array = @arrays[$array_selector];
		
		if ($array eq 'A') {
			my $index = int (rand($a_length));
			$dnt = splice(@a_dnts, $index, 1);
        } elsif ($array eq 'C') {
            my $index = int (rand($c_length));
			$dnt = splice(@c_dnts, $index, 1);
        } elsif ($array eq 'G') {
            my $index = int (rand($g_length));
            $dnt = splice(@g_dnts, $index, 1);
        } elsif ($array eq 'T') {
            my $index = int (rand($t_length));
            $dnt = splice(@t_dnts, $index, 1);
       	} else {
        	$self->throw("Undefined array selector value $array_selector");
	    }
	}

	if (!defined ($dnt)) {
		$self->throw("Could not get a dinucleotide");
	}
	return $dnt;
}
	
