#
# Chum::Neural::Neuron::TransfacNeuron
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Neural::Neuron::TransfacNeuron - A neuron for understanding transfac results from a group

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Neural::Neuron::TransfacNeuron;

use vars qw(@ISA @TN_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
#use Math::SpecFun::Erf qw(erf);

@ISA = qw( Bio::Root::Root );
BEGIN {
        @TN_PARAMS = qw();
        foreach my $attr (@TN_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		$self->_loadTraining();
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub think() {
	my $self = shift;
	my $result = shift;
	
	#print "TESTING: " . $result->{'transfac'}->{'score'} . "\n";
	my $positive_uprob = $self->_probabilityAboveScore($result->{'transfac'}->{'score'}, $self->{'positives'});
	my $neutral_uprob = $self->_probabilityAboveScore($result->{'transfac'}->{'score'}, $self->{'neutrals'});

	my $score = $positive_uprob / ( $positive_uprob + $neutral_uprob);
	#print STDERR $score . "\n";
	if ($score >= $self->{'trained-score'}) {
		return (1, $score - $self->{'trained-score'});
	} else {
		return (0, $score - $self->{'trained-score'});
	}
}

sub _probabilityAboveScore {
	my $self = shift;
	my $score = shift;
	my $results_arrayref = shift;
	my @results = @$results_arrayref;

	my $count = 0;
	foreach my $result (@results) {
		if ($result >= $score) {
			$count++;
		}
	}
	return $count / scalar(@results);
}

sub _probabilityBelowScore {
	my $self = shift;
	my $score = shift;
	my $results_arrayref = shift;
	my @results = @$results_arrayref;

	my $count = 0;
	foreach my $result (@results) {
		if ($result <= $score) {
			$count++;
		}
	}
	return $count / scalar(@results);
}

sub train() {
	my $self = shift;
	my $positiveresults_arrayref = shift;
	my $neutralresultss_arrayref = shift;
	
	my @positives = ();
	my @neutrals = ();
	my @scores = ();
	foreach my $presult (@$positiveresults_arrayref) {
		push @scores, $presult->{'transfac'}->{'score'};
		push @positives, $presult->{'transfac'}->{'score'};
	}
	foreach my $nresult (@$neutralresultss_arrayref) {
		push @scores, $nresult->{'transfac'}->{'score'};
		push @neutrals, $nresult->{'transfac'}->{'score'};
	}
	@scores = sort @scores;
	my @diff = ();
	foreach my $score (@scores) {
		my $pscore = $self->_probabilityBelowScore($score, \@positives);
		my $nscore = $self->_probabilityAboveScore($score, \@neutrals);
		push @diff, abs($pscore - $nscore);
	}
	my $minimum_diff;
	my $minimum_index;
	for (my $i = 0; $i < scalar(@diff); $i++) {
		if (!defined($minimum_diff)) {
			$minimum_diff = @diff[$i];
			$minimum_index = $i;
		} 
		if (@diff[$i] < $minimum_diff) {
			$minimum_diff = @diff[$i];
			$minimum_index = $i;
		}
	}
	my $positive_uprob = $self->_probabilityAboveScore(@scores[$minimum_index], \@positives);
	my $neutral_uprob = $self->_probabilityAboveScore(@scores[$minimum_index], \@neutrals);
	$self->{'trained-score'} = $positive_uprob / ( $positive_uprob + $neutral_uprob);
	print STDERR $self->{'trained-score'} . "\n";
	$self->_saveTraining(\@positives, \@neutrals);
}

sub _saveTraining() {
	my $self = shift;
	my $positives_arrayref = shift;
	my $neutrals_arrayref = shift;
	my @positives = @$positives_arrayref;
	my @neutrals = @$neutrals_arrayref;
	open(TN, ">.transfac.neuron") || $self->throw("Cannot store trained result\n");
	print TN "T:" . $self->{'trained-score'} . "\n";
	foreach my $presult (@positives) {
		print TN "P:" . $presult . "\n";
	}
	foreach my $nresult (@neutrals) {
		print TN "N:" . $nresult . "\n";
	}
	close(TN);
}

sub _loadTraining() {
	my $self = shift;
	if (-e ".transfac.neuron") {
		open(TN, ".transfac.neuron") || $self->throw("Cannot load training data\n");
		my @positives = ();
		my @neutrals = ();
		foreach my $line (<TN>) {
			chomp($line);
			if ($line =~ /P:([\S]+)/g) {
				push @positives, $1;
			} elsif ($line =~ /N:([\S]+)/g) {
				push @neutrals, $1;
			} elsif ($line =~ /T:([\S]+)/g) {
				$self->{'trained-score'} = $1;
			}
		}
		$self->{'positives'} = \@positives;
		$self->{'neutrals'} = \@neutrals;
		close(TN);
	}
}

#$self->{"positive"}->{"mean"} = $self->_calculateTransfacScoreMean(\@positives);
#$self->{"positive"}->{"stdev"} = $self->_calculateTransfacScoreStandardDeviation(\@positives, $self->{"positive"}->{"mean"});
#$self->{"neutral"}->{"mean"} = $self->_calculateTransfacScoreMean(\@neutrals);
#$self->{"neutral"}->{"stdev"} = $self->_calculateTransfacScoreStandardDeviation(\@neutrals, $self->{"neutral"}->{"mean"});

#sub _probabilityPositiveCDF() {  ##The likelihood that this value is in the positive CDF range, we must add extra for values below 0;
#	my $self = shift;
#	my $score = shift;
#	my $positive_mean = shift;
#	my $positive_stdev = shift;
#	my $neutral_mean = shift;
#	my $neutral_stdev = shift;
#
#	my $positive_cdf = $self->_cumulativeDistributionFunction($score, $positive_mean, $positive_stdev);
#	my $neutral_cdf = $self->_cumulativeDistributionFunction($score, $neutral_mean, $neutral_stdev);
#
#	return ($positive_cdf) / ($positive_cdf + $neutral_cdf);
#}
#
#sub _cumulativeDistributionFunction() {
#	my $self = shift;
#	my $value = shift;
#	my $mean = shift;
#	my $stdev = shift;
#	my $cdf = 0.5 * (1 + erf(($value - $mean)/($stdev * sqrt(2)));
#	return $cdf;
#}
#
#sub _calculateTransfacScoreMean() {
#	my $self = shift;
#	my $results_arrayref = shift;
#	my $total = 0;
#	foreach my $results (@$results_arrayref) {
#		$total += $result->{'transfac'}->{'score'};
#	}
#	return $total / scalar(@$results_arrayref);
#}
#
#sub _calculateTransfacScoreStandardDeviation() {
#	my $self = shift;
#	my $results_arrayref = shift;
#	my $mean = shift;
#	if (!defined($mean)) {
#		$mean = $self->_calculateTransfacScoreMean($results_arrayref);
#	}
#	my $total = 0;
#	foreach my $results (@$results_arrayref) {
#		$total += ($result->{'transfac'}->{'score'} - $mean) * ($result->{'transfac'}->{'score'} - $mean);
#	}
#	return sqrt($total / scalar(@$results_arrayref));
#}

1;