#
# Chum::Result::Interpretter::MatrixScannerInterpretter
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::Interpretter::MatrixScannerInterpretter - Interpret result sets from scanning matrix sets derived from matrix databases (like Jaspar)

=head1 SYNOPSIS

=head1 DESCRIPTION

Interpret result sets from Chum::Analysis::Local::MatrixScanner and return a score.
This score looks at the cumulative difference in two lists of predicted transcription
factors using the desired TFBS::MatrixSet.
The scores from each list are cumulated.  The magnitude of the difference in scores is returned by the interpret function.

i.e.
Reference predictions:
Factor1 5
Factor2 6
Factor3 7

Variant predictions:
Factor1 3
Factor2 8
Factor4 4

The returned score would be for Factor1 abs(5-3) = 2 
								Factor2 abs(6-8) = 2
								Factor3 abs(7-0) = 7
								Factor4 abs(0-4) = 4
								returned score = 2+2+7+4= 15

For multiple variants, the maximum score is returned.

Example (scan all PWM's in TransFAC):

my $transfac_db = new Chum::DB::Transfac('transfac_dir' => "$transfac_directory");
$transfac_db->matrixset($transfac_db->_find_all_matrixset());
my $matrix_scanner = new Chum::Analysis::Local::MatrixScanner('matrix_db' => $transfac_db);
my $resultset = $matrix_scanner->run_variation_analysis($stack);
my $msi = new Chum::Result::Interpretter::MatrixScannerInterpretter($resultset);
my $score = $msi->interpret();

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Result::Interpretter::MatrixScannerInterpretter;

use vars qw(@ISA @MSINTERPRETTER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;
use Chum::Result::Interpretter::InterpretterI;

@ISA = qw(Chum::Result::Interpretter::InterpretterI);
BEGIN {
        @MSINTERPRETTER_PARAMS = qw();
        foreach my $attr (@MSINTERPRETTER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;
        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _preload_data {
	#do nothing	
}

=head2 interpret

 Usage   : $interpretter->interpret();
	Calculate the difference score for a MatrixScanner analysis run between
	two sets of hits.  For an example, see Usage at the beginning of this module.
 Returns : integer, score
 Args    : 

=cut

sub interpret {
	my $self = shift;
	return $self->interpret_with_modifiers(undef);
}

=head2 interpret

 Usage   : $interpretter->interpret_with_modifiers($modifiers);
	Calculate the difference score for a MatrixScanner analysis run between
	two sets of hits.  For an example, see Usage at the beginning of this module.
	Results are modified by dividing using the modifier hashref.
 Returns : integer, score
 Args    : Arg1: Hashref (keys are binding site names, values are modifier scores which are divided against raw factor scores)

=cut

sub interpret_with_modifiers() {
	my $self = shift;
	my $modifier_scores_hashref = shift;
	if (!defined($self->resultset)) { warn "Result set undefined"; return -1;}

	my @iresults;
	my %reference_named_scored_hits;
	foreach my $result (@{$self->resultset->results}) {
		my %named_scored_hits;
		foreach my $seqfeature (@{$result->results_object}) {	
			my @scores = split(/:/, $seqfeature->score);
			if (defined($seqfeature->pattern->name)) {
				$named_scored_hits{$seqfeature->pattern->name} += $scores[0];
			} else {
				$named_scored_hits{$seqfeature->pattern->name} = $scores[0];  ##TFBS Names and Cumulative scores
			}
		}
		print STDERR $result->{'type'} . "\n";
		print Dumper \%named_scored_hits;
		my %resultlet = ('named_scored_hits' => \%named_scored_hits, 'result' => $result);
		if ($result->{'type'} eq "CANONICAL") {
			%reference_named_scored_hits = %named_scored_hits;
		}
		push @iresults, \%resultlet;
	}

	my $max_score = 0;
	foreach my $iresult (@iresults) {
		if ($iresult->{'result'}->{'type'} eq "VARIANT") {
			my $variant_named_scored_hits_ref = $iresult->{'named_scored_hits'};
			my %variant_named_scored_hits = %$variant_named_scored_hits_ref;
			my %uberhash = %variant_named_scored_hits;

			for my $key (keys %reference_named_scored_hits) {
				if (!defined($uberhash{$key})) {
					$uberhash{$key} = $reference_named_scored_hits{$key};
				} else {
					$uberhash{$key} = abs($uberhash{$key} - $reference_named_scored_hits{$key});
				}
			}

			##calculate max score
			my $score = 0;
			if (defined ($modifier_scores_hashref)) {
				my %modifier_scores = %$modifier_scores_hashref;
				for my $key (keys %uberhash) {
					if (defined($modifier_scores{$key})) {
						print STDERR "Using modifier for $key: " . $modifier_scores{$key} . "\n";
						$score += $uberhash{$key} / $modifier_scores{$key};
					} else {
						print STDERR "Modifier for $key: NOT FOUND\n";
						$score += $uberhash{$key};
					}
				}
			} else {
				for my $key (keys %uberhash) {
					$score += $uberhash{$key};
				}
			}
			if ($score > $max_score) { $max_score = $score; }
		}
	}
	return $max_score;
}

1;