#
# Chum::Result::Interpretter::ZdnaInterpretter
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::Interpretter::ZdnaInterpretter - Interpret zHunt result sets

=head1 SYNOPSIS

=head1 DESCRIPTION

Interpret zHunt/zDna result sets

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Result::Interpretter::ZdnaInterpretter;

use vars qw(@ISA @ZINTERPRETTER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Bio::Tools::GFF;
use Data::Dumper;
use Chum::Result::Interpretter::InterpretterI;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw(Chum::Result::Interpretter::InterpretterI);
BEGIN {
        @ZINTERPRETTER_PARAMS = qw();
        foreach my $attr (@ZINTERPRETTER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->$attr($value);
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _preload_data {
	my $self = shift;
	foreach my $result (@{$self->resultset->results}) {
		my $filename = $result->results_file;
		my ($tmpfh, $tmpfilename) = tempfile( "zdnaXXXXXX", SUFFIX => '.gff', DIR => "/tmp/", UNLINK => 1);
		$self->resultset->analysis_object->convert_output_to_gff($filename, $tmpfilename, $result->sequence->id);
		my $gffio = Bio::Tools::GFF->new(-file => $tmpfilename, -gff_version => 2);
                my @features;
                while (my $feature = $gffio->next_feature()) {
                        push @features, $feature;
                }
		$result->results_object(\@features);
		unlink0($tmpfh, $tmpfilename) ;
	}
}

sub calculate_maximum_difference {
	my $self = shift;

	my @iresults;
	foreach my $result (@{$self->resultset->results}) {
		my $score = 0;
		my $hits = 0;
		foreach my $seqfeature (@{$result->results_object}) {	##Should be an array of GFF features
			if ($seqfeature->seq_id eq $result->sequence->id) {
				if ($result->sequence->is_start_stop_overlapping($seqfeature->start, $seqfeature->end)) {
					$score += $seqfeature->score;
					$hits++;
					print STDERR "Overlap score " . $score . " " . $result->{'type'} .  " " . $seqfeature->start . " " . $seqfeature->end . "\n";
				}
			}
		}
		my %resultlet = ('hits' => $hits, 'score' => $score, 'result' => $result);
		push @iresults, \%resultlet;
	}

	my $canonical_hits = 0;
	my $canonical_score = 0;
	my $ran_once = 0;
	foreach my $iresult (@iresults) {
		if ($iresult->{'result'}->{'type'} eq "CANONICAL") {
			$canonical_hits = $iresult->{'hits'};
			$canonical_score = $iresult->{'score'};
			$ran_once = 1;
		}
	}
	#if (!$ran_once || $canonical_hits == 0) { $self->throw("No canonical features to analyze");}
	print STDERR "CANONICAL: " . $canonical_score . " " . $canonical_hits . "\n";

	##THIS IS THE MAGNITUDE OF THE DIFFERENCE
	my $max_distance = 0;
	my $total_variant_hits = 0;
	$ran_once = 0;
	foreach my $iresult (@iresults) {
		if ($iresult->{'result'}->{'type'} eq "VARIANT") {
			$ran_once = 1;
			my $variant_hits = $iresult->{'hits'};
			$total_variant_hits += $variant_hits;
			my $variant_score = $iresult->{'score'};
			print STDERR "VARIANT: " . $variant_score . " " . $variant_hits . "\n";
			my $t_max_distance = abs($variant_score - $canonical_score);
			if ($t_max_distance > $max_distance) {
				$max_distance = $t_max_distance;
			}
		}
	}
	#if (!$ran_once || $total_variant_hits == 0) { $self->throw("No variants to analyze");}
	
	return $max_distance;
}

sub calculate_maximum_score {
	my $self = shift;

	my @iresults;
	foreach my $result (@{$self->resultset->results}) {
		my $score = 0;
		my $hits = 0;
		foreach my $seqfeature (@{$result->results_object}) {	##Should be an array of GFF features
			if ($seqfeature->seq_id eq $result->sequence->id) {
				if ($result->sequence->is_start_stop_overlapping($seqfeature->start, $seqfeature->end)) {
					$score += $seqfeature->score;
					$hits++;
					print STDERR "Overlap score " . $score . " " . $result->{'type'} .  " " . $seqfeature->start . " " . $seqfeature->end . "\n";
				}
			}
		}
		my %resultlet = ('hits' => $hits, 'score' => $score, 'result' => $result);
		push @iresults, \%resultlet;
	}

	my $canonical_hits = 0;
	my $canonical_score = 0;
	my $ran_once = 0;
	foreach my $iresult (@iresults) {
		if ($iresult->{'result'}->{'type'} eq "CANONICAL") {
			$canonical_hits = $iresult->{'hits'};
			$canonical_score = $iresult->{'score'};
			$ran_once = 1;
		}
	}
	#if (!$ran_once || $canonical_hits == 0) { $self->throw("No canonical features to analyze");}
	print STDERR "CANONICAL: " . $canonical_score . " " . $canonical_hits . "\n";

	##THIS IS THE MAGNITUDE OF THE DIFFERENCE
	my $max_score = $canonical_score;
	my $total_variant_hits = 0;
	$ran_once = 0;
	foreach my $iresult (@iresults) {
		if ($iresult->{'result'}->{'type'} eq "VARIANT") {
			$ran_once = 1;
			my $variant_hits = $iresult->{'hits'};
			$total_variant_hits += $variant_hits;
			my $variant_score = $iresult->{'score'};
			
			print STDERR "VARIANT: " . $variant_score . " " . $variant_hits . "\n";
			
			if ($variant_score > $max_score) {
				$max_score = $variant_score;
			}
		}
	}
	#if (!$ran_once || $total_variant_hits == 0) { $self->throw("No variants to analyze");}
	
	return $max_score;
}

1;