#
# Chum::Stack
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Stack - An analysis stack object

=head1 SYNOPSIS

=head1 DESCRIPTION

A set of orthologous sequences, and reference and variant sequences with their associated alleles inserted.
An option variation_ref_obj parameter provides metadata about the stack.  It can be either a string or a Bio::EnsEMBL::Variation::VariationFeature object

Example: 

my $stack = new Chum::Stack(	'reference_sequence' => $reference_sequence, ##A Chum::VariantSequence object
				'orthologous_sequences' => \@orthologous_sequences, ##An arrayref of Bio::Seq or Bio::LocatableSeq objects
				'variant_sequences' => \@variant_sequences, ##An arrayref of Chum::VariantSequence objects
				'variant_ref_obj' => $variation_feature); 

The essential difference between the reference sequence and the variant sequence is that the reference sequence is located
in a canonical genome assembly.  The stack object can be used to test various sequence combinations
for allele specific results.  For instance as input to an analysis program, the reference sequence will be run
against various variant_sequences to determine the effects an allele has on the results of a particular analysis.

The easiest way of creating new Chum::Stack objects is through the Chum::Utils::RSNPAnalysisXMLFileReader
They can also be created by direct calls to EnsEMBL or from appropriately formatted GFF files (which can
gather sequence from EnsEMBL).

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Stack;

use vars qw(@ISA @STACK_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @STACK_PARAMS = qw(reference_sequence orthologous_sequences variant_sequences variation_ref_obj);
        foreach my $attr (@STACK_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		if (!defined($self->{'reference_sequence'})) { $self->throw("Error: no reference_sequence defined"); }
		unless ($self->{'reference_sequence'}->isa("Chum::VariantSequence")) { $self->throw("Error: reference_sequence not a Chum::VariantSequence object"); }
		if (!defined($self->{'orthologous_sequences'})) { $self->throw("Error: No orthologous_sequences defined"); }
		foreach my $orthologous_sequence (@{$self->{'orthologous_sequences'}}) {
			unless ($orthologous_sequence->isa("Bio::Seq") || $orthologous_sequence->isa("Bio::LocatableSeq")) { $self->throw("Error: orthologous sequence not a Bio::Seq object in orthologous_sequences array"); }
		}
		if (!defined($self->{'variant_sequences'})) { die "Error: No variant_sequences defined"; }
		foreach my $variant_sequence (@{$self->{'variant_sequences'}}) {
			unless ($variant_sequence->isa("Chum::VariantSequence")) { $self->throw("Error: variant sequence not a Chum::VariantSequence object in variant_sequences array"); }
		}
		##variation_ref_obj can be either a string or a Bio::EnsEMBL::Variation::VariationFeature object, 
		##it is designed to contain meta information about the variation associated to this stack object
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

=head2 reference_sequence

 Usage   : $stack->reference_sequence;
		   the variant sequence that is present in a canonical genome assembly (i.e. human), it is not
		   present in the orthologous sequences array and should be added when performing tasks
		   such as "phylogenetic footprinting"
 Returns : Chum::VariantSequence
 Args    : none

=cut

sub reference_sequence {
	my $self = shift;
	return $self->{'reference_sequence'};
}

=head2 orthologous_sequences

 Usage   : my @orthologous_sequences = @{$stack->orthologous_sequences};
		   sequences identified as orthologous to the reference sequence
 Returns : an arrayref of Bio::Seq or Bio::LocatableSeq objects
 Args    : none

=cut

sub orthologous_sequences {
	my $self = shift;
	return $self->{'orthologous_sequences'};
}

=head2 variant_sequences

 Usage   : my @variant_sequences = @{$stack->variant_sequences};
		   sequences with variant alleles inserted, typically not in a canonical genome assembly
 Returns : an arrayref of Chum::VariantSequence objects
 Args    : none

=cut

sub variant_sequences {
	my $self = shift;
	return $self->{'variant_sequences'};
}

=head2 variation_ref_obj

 Usage   : $stack->variation_ref_obj;
		   An optional meta data object containing information on the variation this stack object is
		   associated to.
 Returns : string or a Bio::EnsEMBL::Variation::VariationFeature
 Args    : none

=cut

sub variation_ref_obj {
	my $self = shift;
	return $self->{'variation_ref_obj'};
}

=head2 get_variation_name

 Usage   : $stack->get_variation_name;
		   The name of the variant, i.e. "rs1234567" (dbSNP)
 Returns : string
 Args    : none

=cut

sub get_variation_name {
	my $self = shift;
	if ($self->variation_ref_obj->isa('Bio::EnsEMBL::Variation::VariationFeature')) {
		return $self->variation_ref_obj->display_id;
	} else {
		return $self->variation_ref_obj;
	}
}

=head2 get_orthologous_sequence_ids

 Usage   : my @ids = @{$stack->get_orthologous_sequence_ids};
		   The display ids of the orthologous sequences.
 Returns : an arrayref of strings
 Args    : none

=cut

sub get_orthologous_sequence_ids {
	my $self = shift;
	my @ids;
	foreach  my $seq (@{$self->orthologous_sequences}) {
		push @ids, $seq->id;
	}
	return \@ids;
}

=head2 get_orthologous_sequence_by_id

 Usage   : my $chimp_seq_obj = $stack->get_orthologous_sequence_by_id("patr");
		   T
 Returns : A Bio::Seq or Bio::LocatableSeq object
 Args    : arg1: the id of the orthologous sequence to return

=cut


sub get_orthologous_sequence_by_id {
	my $self = shift;
	my $id = shift;
	if (!defined($id)) { return undef; }
	
	foreach  my $seq (@{$self->orthologous_sequences}) {
		if (lc($id) eq lc($seq->id)) {
			return $seq;	
		}
	}
	return undef;
}

=head2 get_sequence_ids_string

 Usage   : $stack->get_sequence_ids_string
		   The display ids of the sequences with their sequence lengths
		   Simple way of outputting a stacks contents to the display.
 Returns : string
 Args    : none

=cut

sub get_sequence_ids_string {
	my $self = shift;
	my $id_string = $self->reference_sequence->id . " (" . length($self->reference_sequence->sequence->seq)  . ")";
	my @stack = @{$self->orthologous_sequences};
	for (my $i = 0; $i < scalar(@stack); $i++) {
		if ($i == 0) { $id_string .= ", "; }
		if ($i < (scalar(@stack) - 1)) {
			$id_string .= $stack[$i]->id . " (" . length($stack[$i]->seq) . "), ";
		} else {
			$id_string .= $stack[$i]->id . " (" . length($stack[$i]->seq) . ")";
		}
	}
	return $id_string;
}

1;
