#
# Chum::Statistics::Gist::GistResult
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod
=head1 NAME
Chum::Statistics::Gist::GistResult - A Gist results object
=head1 SYNOPSIS
=head1 DESCRIPTION
=head1 CONTACT
Stephen Montgomery (smontgom@bcsgc.bc.ca)
=head1 METHODS
=cut

package Chum::Statistics::Gist::GistResult;

use vars qw(@ISA @GRESULT_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @GRESULT_PARAMS = qw(roc_file score_file partition_number type_name);
    foreach my $attr (@GRESULT_PARAMS)
        { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
        my $attr = shift @args;
        my $value = shift @args;
        $self->{$attr} = $value;
    }
    if (!defined($self->{'roc_file'})) { $self->throw("No 'roc_file' specified"); }
    unless (-e $self->{'roc_file'}) { $self->throw("'roc_file' does not exist"); }
    if (!defined($self->{'score_file'})) { $self->throw("No 'score_file' specified"); }
    unless (-e $self->{'score_file'}) { $self->throw("'score_file' does not exist"); }
        
    if ($self->{'roc_file'} =~ /[\S\s]+partition.(test|train).([\S]+).([0-9]+).roc/g) {
        $self->{'type_name'} = $2;
        $self->{'partition_number'} = $3;
    } else {
        $self->warn("Could not set 'partition_number' or 'type_name' from 'roc_file' name");   
    }

    $self->_load_roc_file();
    $self->_load_score_file();
        
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub _load_roc_file() {
    my $self = shift;   
    open (ROC, $self->{'roc_file'}) || $self->throw("Could not open 'roc_file': " . $self->{'roc_file'} );
    my ($test_threshold_fp, $test_threshold_tp, @fp, @tp);
    my $lineindex = 0;
    while (my $line = <ROC>) {
        chomp($line);
        if ($lineindex++ == 0) {
            if ($line =~ /[\S]+[\s]+([0-9]+)[\s]+([0-9]+)/) {
                $test_threshold_fp = $1;
                $test_threshold_tp = $2;
            } else {
                $self->throw("Could not find threshold ROC values for 'roc_file':" . $self->{'roc_file'});
            }
        } else {
            if ($line =~ /([0-9]+)[\s]+([0-9]+)/) {
                push @fp, $1;
                push @tp, $2;
            }
        }
    }
    close(ROC);
    
    ####CALCULATE TN and FN rates
    my $total_n = $fp[scalar(@fp) - 1];
    my $total_p = $tp[scalar(@tp) - 1];
    my (@fn, @tn) = ();
    for (my $i = 0; $i < scalar(@tp); $i++) {
        my $tn_i = $total_n - $fp[$i];
        push @tn, $tn_i;
        my $fn_i = $total_p - $tp[$i];
        push @fn, $fn_i;
    }
    
    ####CALCULATE SENSITIVITY/SPECIFICITY
    my (@sensitivity, @specificity) = ();
    for (my $i = 0; $i < scalar(@tp); $i++) {
        my $sens_i = $tp[$i] / ($tp[$i] + $fn[$i]);
        push @sensitivity, $sens_i;
        #print $tp[$i] . "/(" . $tp[$i] . "+" . $fn[$i] . ") = " . $sens_i . "\n";
        my $spec_i = $tn[$i] / ($tn[$i] + $fp[$i]);
        push @specificity, $spec_i;
    }
    
    $self->{'tp'} = \@tp;
    $self->{'fp'} = \@fp;
    $self->{'tn'} = \@tn;
    $self->{'fn'} = \@fn;
    $self->{'test_threshold_tp'} = $test_threshold_tp;
    $self->{'test_threshold_fp'} = $test_threshold_fp;
    $self->{'test_threshold_tn'} = $total_n - $test_threshold_fp;
    $self->{'test_threshold_fn'} = $total_p - $test_threshold_tp;
    $self->{'test_threshold_sensitivity'} = $self->{'test_threshold_tp'} / ($self->{'test_threshold_tp'} + $self->{'test_threshold_fn'});
    $self->{'test_threshold_specificity'} = $self->{'test_threshold_tn'} / ($self->{'test_threshold_tn'} + $self->{'test_threshold_fp'});
    $self->{'p'} = $total_p;
    $self->{'n'} = $total_n;
    $self->{'sensitivity'} = \@sensitivity;
    $self->{'specificity'} = \@specificity;
    
    return 0;
}

sub _load_score_file() {
    my $self = shift;
    open(SCORE, $self->{'score_file'}) || $self->throw("Could not open 'score_file': " . $self->{'score_file'} );
    my $lineindex = 0;
    while (my $line = <SCORE>) {
        chomp($line);
        if ($lineindex == 0) {
            if ($line =~ /Number[\s\S]+training[\s\S]+:[\s]+([0-9]+)[\s]+[\S]{1}([0-9]+)[\s]+
                        positive[\S]{1}[\s]+([0-9]+)[\s]+negative[\S\s]+/xg) {
                $self->{'training_examples'} = $1;
                $self->{'positive_training_examples'} = $2;
                $self->{'negative_training_examples'} = $3;
            } else {
                print STDERR $line . "\n";
                $self->throw("Could not get number of training examples from 'score_file': " . $self->{'score_file'});   
            }
        } elsif ($lineindex == 1) {
            if ($line =~ /Number[\s\S]+support[\s\S]+:[\s]+([0-9]+)[\s]+[\S]{1}([0-9]+)[\s]+
                        positive[\S]{1}[\s]+([0-9]+)[\s]+negative[\S]+[\s]+([\S]+)/xg) {
                $self->{'support_vectors'} = $1;
                $self->{'positive_support_vectors'} = $2;
                $self->{'negative_support_vectors'} = $3;
                $self->{'percentage_support_vectors'} = $4;
            } else {
                print STDERR $line . "\n";
                $self->throw("Could not get number of support vectors from 'score_file': " . $self->{'score_file'});   
            }   
        } elsif ($lineindex == 2) {
            if ($line =~ /Training[\s]+results:[\s]+FP=([0-9]+)[\s]+FN=([0-9]+)[\s]+
                        TP=([0-9]+)[\s]+TN=([0-9]+)/xg) {
                $self->{'training_fp'} = $1;
                $self->{'training_fn'} = $2;
                $self->{'training_tp'} = $3;
                $self->{'training_tn'} = $4;
            } else {
                print STDERR $line . "\n";
                $self->throw("Could not get training results from 'score_file': " . $self->{'score_file'});   
            }   
        } elsif ($lineindex == 3) {
            if ($line =~ /Training[\s]+ROC:[\s]+([\S]+)/xg) {
                $self->{'training_roc'} = $1;
            } else {
                print STDERR $line . "\n";
                $self->throw("Could not get training ROC from 'score_file': " . $self->{'score_file'});   
            }   
        } elsif ($lineindex == 4) {
            if ($line =~ /Test[\s]+results:[\s]+FP=([0-9]+)[\s]+FN=([0-9]+)[\s]+
                        TP=([0-9]+)[\s]+TN=([0-9]+)/xg) {
                $self->{'test_fp'} = $1;
                $self->{'test_fn'} = $2;
                $self->{'test_tp'} = $3;
                $self->{'test_tn'} = $4;
            } else {
                print STDERR $line . "\n";
                $self->throw("Could not get test results from 'score_file': " . $self->{'score_file'});   
            }   
        } elsif ($lineindex == 5) {
            if ($line =~ /Test[\s]+ROC:[\s]+([\S]+)/xg) {
                $self->{'test_roc'} = $1;
            } else {
                print STDERR $line . "\n";
                $self->throw("Could not get test ROC from 'score_file': " . $self->{'score_file'});   
            }   
        }
        $lineindex++;
    }
    close(SCORE);
    
    return 0;
}
