package Chum::Utils::AlleleMapper;

use vars qw(@ISA @AM_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Factory::EMBOSS;
use Bio::Root::Root;
use Bio::Seq;
require Chum::DB::Oreganno;
require Chum::Utils::GenomeCoordinateLocator;
use Data::Dumper;
use File::Temp qw/ tempfile tempdir unlink0/;
use SOAP::Lite;

@ISA = qw( Bio::Root::Root );

BEGIN {
    @AM_PARAMS = qw();
    foreach my $attr (@AM_PARAMS)
        { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
        my $attr = shift @args;
        my $value = shift @args;
        $self->$attr($value);
    }
    
    my $BLAST_DB_FILE = "hg17.fa";
    $self->{'sa'} = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Slice");
    $self->{'va'} = Bio::EnsEMBL::Registry->get_adaptor("human", "variation", "Variation");
    $self->{'vfa'} = Bio::EnsEMBL::Registry->get_adaptor("human", "variation", "VariationFeature");
    $self->{'gcl'} = new Chum::Utils::GenomeCoordinateLocator('blast_db' => $BLAST_DB_FILE);
    
    return $self;
}
sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

##arrayref of hash objects containing start, stop, chr, allele, 5flank, 3flank
sub get_mapped_polymorphism_sequences_for_oreganno_id() {
    my $self = shift;
    my $oreganno_id = shift;
    if (!defined($oreganno_id)) { $self->throw("No ORegAnno ID has been defined"); }
    unless ($oreganno_id =~ /OREG[0-9]+/g) {
        $self->throw("ORegAnno ID is invalid: " . $oreganno_id);   
    }
    my $oreganno_db = new Chum::DB::Oreganno();
    my $record = $oreganno_db->get_record_from_id($oreganno_id);
    if (!defined($record)) {$self->throw("Could not find ORegAnno record for " . $oreganno_id); }
    else {
        my @return_mapped_allele_sequences;
        my $variations_arrayref = $oreganno_db->get_variations_arrayref_from_regulatory_variant_record($record);
        if (defined($variations_arrayref)) {
            foreach my $variation (@$variations_arrayref) {
                my $loaded = 0;
                
                ##1. TRY DBSNP
                my $dbsnp_id = $oreganno_db->get_dbsnp_id_from_variation($variation);
                if (defined($dbsnp_id)) {
                    my $mapped_allele_sequence = undef;
                    eval {
                        $mapped_allele_sequence = $self->get_mapped_polymorphism_sequences_for_dbsnp_id($dbsnp_id);
                    };
                    if ($@) {
                        print STDERR $@;   
                    }
                    if (defined($mapped_allele_sequence)) {
                        push @return_mapped_allele_sequences, $mapped_allele_sequence;
                        $loaded = 1;
                    }
                }
                
                ##2. TRY INTERNAL, IF DBSNP DOESN'T WORK
                if (!$loaded) {
                    my $h_chr = undef;
                    my $h_start = undef;
                    my $h_end = undef;
                    my $h_strand = undef;
                    my $reference_sequence = $variation->{'referenceSequence'};   
                    my $hits_arrayref = $self->{'gcl'}->locate_sequence($oreganno_id, $reference_sequence);
                    if (defined($hits_arrayref)) {
                        my @hits = @$hits_arrayref;
                        if (scalar(@hits) == 1) {
                            my $hit = $hits[0];
                            $h_chr = $hit->{'chromosome'};
                            $h_start = $hit->{'start'};
                            $h_end = $hit->{'end'};
                            $h_strand = $hit->{'strand'};
                        }
                    }
                    if (!defined($h_chr) || !defined($h_start) || !defined($h_end) || !defined($h_strand)) {
                        $self->throw("Could not map reference sequence for variant: " . $oreganno_id . "\n" . $reference_sequence);
                    } else {
			print STDERR "Using hit at " . $h_chr . " " . $h_start . " " . $h_end . " " . $h_strand . "\n";
		    }
                
                    ##should have unique location of blast hit, now must compare all alleles to determine what the allele position is
                    my $variant_sequence = $variation->{'variantSequence'};
                    
                    my $allelesref = $self->get_alleles($reference_sequence, $variant_sequence);
                    if (!defined($allelesref)) {
                        $self->throw("Could not determine alleles for variant: " . $oreganno_id);
                    } else {
                        my @alleles = @$allelesref;
                        if (scalar(@alleles) >= 2) {
                            my $refallele = undef;
                            my @seqalleles = ();
                            foreach my $allele (@alleles) {
                                if (lc($allele->{'seq_5'} . $allele->{'allele'} . $allele->{'seq_3'}) eq lc($reference_sequence) ) {
                                    $refallele =  $allele;   
                                }
                                push @seqalleles, $allele->{'allele'};
                            }
                            if (defined($refallele)) {
                                my $length;
                                if ($h_strand eq "+" || $h_strand eq "+1" || $h_strand == 1) {
                                    $length = length($refallele->{'seq_5'});
                                } elsif ($h_strand eq "-" || $h_strand eq "-1" || $h_strand == -1) {
                                    $length = length($refallele->{'seq_3'});
                                } else {
                                    $self->throw("Unidentified strand from blast: " . $h_strand);
                                }
                                my $varlength = length($refallele->{'allele'}); #ERROR HERE!!
                                my $start = $h_start + $length;
                                my $end = $varlength + $start - 1;
                                
                                my $mapped_polymorphism_sequence = {    'seq_5' => $refallele->{'seq_5'},
                                                                        'seq_3' => $refallele->{'seq_3'},
                                                                        'chr' => $h_chr,
                                                                        'start' => $start,
                                                                        'end' => $end,
                                                                        'strand' => $h_strand,
                                                                        'alleles' => \@seqalleles};
                                push @return_mapped_allele_sequences, $mapped_polymorphism_sequence;         
                            }
                        } else {
                            $self->throw("Could find distinct alleles for: " . $oreganno_id);   
                        }
                    }
                }
            }
        }
        return \@return_mapped_allele_sequences;
    }
}

sub get_mapped_polymorphism_sequences_for_dbsnp_id() {
    my $self = shift;
    my $dbsnp_id = shift;
    if (!defined($dbsnp_id)) { $self->throw("No dbSNP ID has been defined"); }
    unless ($dbsnp_id =~ /rs[0-9]+/g) {
        $self->throw("dbSNP ID is invalid: " . $dbsnp_id);   
    }
    my $variation = $self->{'va'}->fetch_by_name($dbsnp_id, 'dbSNP');
    my @all_alleles = @{$variation->get_all_Alleles()};
    my @alleles = ();
    my %allelehash;
    foreach my $allele (@all_alleles) {
	$allelehash{$allele->allele} = 1;
    }
    @alleles = keys %allelehash;
    my @variation_features = @{$self->{'vfa'}->fetch_all_by_Variation($variation)};
    if (scalar(@variation_features) == 1) {
        my $variation_feature = $variation_features[0];
        $variation_feature = $variation_feature->transform('toplevel');
        my $seq_region_name = $variation_feature->seq_region_name;
        my $start = $variation_feature->start;
        my $end = $variation_feature->end;
        my $strand = $variation_feature->strand;
        my @ret_alleles = ();
        if ($strand eq "1") {
            my $slice_5 = $self->{'sa'}->fetch_by_region('chromosome', $seq_region_name,  $start - 51, $start - 1, 1);
            my $slice_3 = $self->{'sa'}->fetch_by_region('chromosome', $seq_region_name,  $end + 1, $end + 51, 1);
            my $seq_5 = $slice_5->seq;
            my $seq_3 = $slice_3->seq;
            foreach my $allele (@alleles) {
                push @ret_alleles, $allele;
            }
            my $mapped_polymorphism_sequence = { 'seq_5' => $seq_5,
                                                 'seq_3' => $seq_3,
                                                 'chr' => $seq_region_name,
                                                 'start' => $start,
                                                 'end' => $end,
                                                 'strand' => $strand,
                                                 'alleles' => \@ret_alleles};
	    return $mapped_polymorphism_sequence;
        } elsif ($strand eq "-1") {
            my $slice_5 = $self->{'sa'}->fetch_by_region('chromosome', $seq_region_name,  $end + 1, $end + 51, -1);
            my $slice_3 = $self->{'sa'}->fetch_by_region('chromosome', $seq_region_name,  $start - 51, $start - 1, -1);
            my $seq_5 = $slice_5->seq;
            my $seq_3 = $slice_3->seq;
            foreach my $allele (@alleles) {
                #my $allele_sobj = new Bio::Seq(-id => $dbsnp_id, -seq => $allele);
                #push @ret_alleles, $allele_sobj->revcom->seq;
		push @ret_alleles, $allele;
            }
            my $mapped_polymorphism_sequence = { 'seq_5' => $seq_5,
                                                 'seq_3' => $seq_3,
                                                 'chr' => $seq_region_name,
                                                 'start' => $start,
                                                 'end' => $end,
                                                 'strand' => $strand,
                                                 'alleles' => \@ret_alleles};
	    return $mapped_polymorphism_sequence;
        } else {
            $self->throw("Unknown strand: " . $strand);
        }
    } else {
        $self->throw("Could not find unique genome mapping for dbSNP ID: " . $dbsnp_id);   
    }
    return undef;
}

##############################################ALLELES SUBROUTINES

sub get_alleles() { ##Finds ORegAnno formatted alleles, where both the reference and variant sequence alleles are uppercase
    my $self = shift;
    my $reference_seq = shift;
    my $variant_seq = shift;
    my @alleles;
    if ($reference_seq =~ /[atcg]+([ATCG]+)[atcg]+/g) {
        my $reference_allele = $1;
        if ($variant_seq =~ /[atcg]+([ATCG]+)[atcg]+/g) {
            my $variant_allele = $1;
            my $ref_index = index($reference_seq, $reference_allele);
            my $left_sequence = substr($reference_seq, 0, $ref_index);
            my $right_sequence = substr($reference_seq, $ref_index + length($reference_allele));
            push @alleles, {'seq_5' => $left_sequence,
                            'seq_3' => $right_sequence,
                            'allele' => lc($reference_allele)};
            push @alleles, {'seq_5' => $left_sequence,
                            'seq_5' => $right_sequence,
                            'allele' => lc($variant_allele)};
            return \@alleles;
        }
    }
    return $self->get_alleles_using_diffseq(lc($reference_seq), lc($variant_seq));
}

sub get_alleles_using_diffseq() {
    my $self = shift;
    my $reference_seq = shift;
    my $variant_seq = shift;
    unless(defined($reference_seq) && defined($variant_seq)) { $self->throw("No sequences entered for diffseq function\n"); }
    
    my $ref_seqobj = new Bio::Seq(-id => 'reference', -seq => lc($reference_seq));
    my $var_seqobj = new Bio::Seq(-id => 'variant', -seq => lc($variant_seq));
    
    my $factory = new Bio::Factory::EMBOSS;
    my ($tmpfh, $tmpfile) = tempfile("diffseq.tmpXXXXX", SUFFIX => '.out');
    my ($atmpfh, $atmpfile) = tempfile("diffseq.atmpXXXXX", SUFFIX => '.gff');
    my ($btmpfh, $btmpfile) = tempfile("diffseq.btmpXXXXX", SUFFIX => '.gff');
    $factory->{'_programs'}->{'diffseq'} = 1; ##Required on windows systems, just use 'diffseq' on linux
    my $diffseqapp = $factory->program('diffseq');
    my %input = (   -asequence => $ref_seqobj,
                    -bsequence => $var_seqobj,
                    -outfile  =>  $tmpfile,
                    -aoutfeat => $atmpfile,
                    -boutfeat => $btmpfile);
    $diffseqapp->run(\%input);
    my $agffio = Bio::Tools::GFF->new(-file => $atmpfile, -gff_version => 2);
    
    my @alleles = ();
    while (my $feature = $agffio->next_feature()) {
        print ".";
        my $ref_left_sequence = substr($reference_seq, 0, $feature->start - 1);
        my $ref_right_sequence = substr($reference_seq, $feature->end);
        my $refallele = substr($reference_seq, $feature->start - 1, $feature->end - $feature->start + 1);
        my @annotations = $feature->annotation->get_Annotations('replace');
        my $varallele = $annotations[0]->value;
        push @alleles, {
                        'seq_5' => $ref_left_sequence,
                        'seq_3' => $ref_right_sequence,
                        'allele' => $refallele};
        if ($varallele eq '') { push @alleles, {
                                                'seq_5' => $ref_left_sequence,
                                                'seq_3' => $ref_right_sequence,
                                                'allele' => ''}; }
    }
    my $bgffio = Bio::Tools::GFF->new(-file => $btmpfile, -gff_version => 2);
    while (my $feature = $bgffio->next_feature()) {
        print ".";
        my $ref_left_sequence = substr($variant_seq, 0, $feature->start - 1);
        my $ref_right_sequence = substr($variant_seq, $feature->end);
        my $refallele = substr($variant_seq, $feature->start - 1, $feature->end - $feature->start + 1);
        my @annotations = $feature->annotation->get_Annotations('replace');
        my $varallele = $annotations[0]->value;
        print STDERR "Found alleles: 1-" . $refallele . "\t2-" . $varallele . "\n";
        if ($varallele eq '') { push @alleles, {
                                                'seq_5' => $ref_left_sequence,
                                                'seq_3' => $ref_right_sequence,
                                                'allele' => ''};}
        push @alleles, {
                        'seq_5' => $ref_left_sequence,
                        'seq_3' => $ref_right_sequence,
                        'allele' => $refallele};
    }
    
    unlink0($tmpfh, $tmpfile);
    unlink0($atmpfh, $atmpfile);
    unlink0($btmpfh, $btmpfile);
    return \@alleles;
}


