package Chum::Utils::AnalysisExpectancyCalculator;

use vars qw(@ISA @AEC_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Analysis::ExtData::CpGIsland;
use Chum::Analysis::ExtData::NhgriDNaseISite;
use Chum::Analysis::ExtData::NobleDNaseISite;
use Chum::Analysis::ExtData::PhastCons;
use Chum::Analysis::ExtData::RegulatoryPotential;
use Chum::Utils::RepeatContentFinder;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@AEC_PARAMS = qw(coord_file chromosomes_arrayref);
	foreach my $attr (@AEC_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
	}
	if (!defined($self->{'coord_file'})) { $self->throw("coord_file not inputted to object"); }
	else {
		if (!defined($self->{'chromosomes_arrayref'})) {
			my @CHROMOSOMES = ("1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20","21","22","X");
			$self->{'chromosomes_arrayref'} = \@CHROMOSOMES;
		}
		
		$self->_load_coord_file($self->{'coord_file'});
	}
	return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub _load_coord_file {
	my $self = shift;
	my $coord_file = shift;
	if (!defined($coord_file)) { $self->throw("Coord file not defined"); }
	unless (-f $coord_file) { $self->throw("Coord file does not exist"); }
	open(COORD, $coord_file) || $self->throw("Cannot open coord file");
	my @values = ();
	
	my %chr_ref;
	if (defined($self->{'chromosomes_arrayref'})) {
		foreach my $chromosome (@{$self->{'chromosomes_arrayref'}}) {
			$chr_ref{$chromosome} = 1;	
		}
	} else {
		$self->throw("No chromosomes to analyze");	
	}
	
	
	my %COORDHASH;
	while (my $line = <COORD>) {
		chomp($line);
		if ($line =~ /[\S]{1}INTERVAL:[\s]+([\S]+)/g) { $self->{'interval'} = $1;
		} elsif ($line =~ /[\S]{1}LENGTH:[\s]+([\S]+)/g) { $self->{'length'} = $1;
		} elsif ($line =~ /[\S]{1}NUMBEROFGENES:[\s]+([\S]+)/g) { $self->{'number_of_genes'} = $1;
		} elsif ($line =~ /[\S]{1}SAMPLES:[\s]+([\S]+)/g) { $self->{'samples'} = $1;
		} elsif ($line =~ /CHROMOSOME[\s]+TRANSCRIPT[\s]+([\S\s]+)/g) {
			my $cline = $1;
			@values = split '\t', $cline;
		} elsif ($line =~ /([\S]+)[\s]+([\S]+)[\s]+([\S\s]+)/g) {
			my $chromosome = $1;
			if ($chr_ref{$chromosome} == 1) {
				my $transcript = $2;
				my $cline = $3;
				my @carray = ();
				my @coords = split '\t', $cline;
				foreach my $coord (@coords) {
					my @coords_xy = split ':', $coord;
					my %coord_hash = ( 'start' => $coords_xy[0],
							   'end' => $coords_xy[1]);
					push @carray, \%coord_hash;
				}
				if (!defined($COORDHASH{$chromosome}->{$transcript})) {
					$COORDHASH{$chromosome}->{$transcript} = \@carray;	
				} else {
					$self->throw("Already found $transcript");	
				}
			}
		}
	}
	close(COORD);
	$self->{'coord_hash'} = \%COORDHASH;
}

sub calculate_cpg_expectancies_hashref {
	my $self = shift;
	my $cpg_detector = new Chum::Analysis::ExtData::CpGIsland();
	my %score_hash;
	foreach my $chromosome (keys %{$self->{'coord_hash'}}) {
		my @transcripts = keys %{$self->{'coord_hash'}->{$chromosome}};
		for (my $i = 0; $i < scalar(@transcripts); $i++) {
			my @coordinates_array = @{$self->{'coord_hash'}->{$chromosome}->{$transcripts[$i]}};	
			for (my $j = 0; $j < scalar(@coordinates_array); $j++) {
				eval {	
					my $score = $cpg_detector->in_cpg($chromosome, $coordinates_array[$j]->{'start'}, $coordinates_array[$j]->{'end'});
					$score_hash{$chromosome}{$transcripts[$i]}{$j} = $score;
				};
				if ($@) {
					print STDERR $@;	
				}
			
			}
		}
	}
	return \%score_hash;
}

sub calculate_nhgri_dnase_expectancies_hashref {
	my $self = shift;
	my $nhgri_dnase_site_detector = new Chum::Analysis::ExtData::NhgriDNaseISite();
	my %score_hash;
	foreach my $chromosome (keys %{$self->{'coord_hash'}}) {
		my @transcripts = keys %{$self->{'coord_hash'}->{$chromosome}};
		for (my $i = 0; $i < scalar(@transcripts); $i++) {
			my @coordinates_array = @{$self->{'coord_hash'}->{$chromosome}->{$transcripts[$i]}};	
			for (my $j = 0; $j < scalar(@coordinates_array); $j++) {
				eval {	
					my $score = $nhgri_dnase_site_detector->in_dnase1_hs_site($chromosome, $coordinates_array[$j]->{'start'}, $coordinates_array[$j]->{'end'});
					$score_hash{$chromosome}{$transcripts[$i]}{$j} = $score;
				};
				if ($@) {
					print STDERR $@;	
				}
			
			}
		}
	}
	return \%score_hash;
}

sub calculate_noble_dnase_expectancies_hashref {
	my $self = shift;
	my $noble_dnase_site_detector = new Chum::Analysis::ExtData::NobleDNaseISite('_load' => 1);
	my %score_hash;
	foreach my $chromosome (keys %{$self->{'coord_hash'}}) {
		my @transcripts = keys %{$self->{'coord_hash'}->{$chromosome}};
		for (my $i = 0; $i < scalar(@transcripts); $i++) {
			my @coordinates_array = @{$self->{'coord_hash'}->{$chromosome}->{$transcripts[$i]}};	
			for (my $j = 0; $j < scalar(@coordinates_array); $j++) {
				eval {	
					my $score = $noble_dnase_site_detector->in_dnase1_hs_site($chromosome, $coordinates_array[$j]->{'start'}, $coordinates_array[$j]->{'end'});
					$score_hash{$chromosome}{$transcripts[$i]}{$j} = $score;
				};
				if ($@) {
					print STDERR $@;	
				}
			
			}
		}
	}
	return \%score_hash;
}

sub calculate_phastcons_expectancies_hashref {
	my $self = shift;
	my $phastcons_detector = new Chum::Analysis::ExtData::PhastCons('phast_cons_dir' => "/home/pubseq/databases/phast_cons/mzPt1Mm5Rn3Cf1Gg2Fr1Dr1",
									'_cache_on' => 1);
	
	my %score_hash;
	foreach my $chromosome (keys %{$self->{'coord_hash'}}) {
		my @transcripts = keys %{$self->{'coord_hash'}->{$chromosome}};
		for (my $i = 0; $i < scalar(@transcripts); $i++) {
			my @coordinates_array = @{$self->{'coord_hash'}->{$chromosome}->{$transcripts[$i]}};	
			for (my $j = 0; $j < scalar(@coordinates_array); $j++) {
				eval {	
					my $score = $phastcons_detector->average_phast_cons_score($chromosome, $coordinates_array[$j]->{'start'}, $coordinates_array[$j]->{'end'});
					$score_hash{$chromosome}{$transcripts[$i]}{$j} = $score;
				};
				if ($@) {
					print STDERR $@;	
				}
			
			}
		}
	}
	return \%score_hash;
}

sub calculate_regulatory_potential_expectancies_hashref {
	my $self = shift;
	my $rp_detector = new Chum::Analysis::ExtData::RegulatoryPotential('regulatory_potential_dir' => "/home/pubseq/databases/regulatory_potential",
									   '_cache_on' => 1,
									   '_big_cache_on' => 1);
	
	my %score_hash;
	foreach my $chromosome (keys %{$self->{'coord_hash'}}) {
		my @transcripts = keys %{$self->{'coord_hash'}->{$chromosome}};
		for (my $i = 0; $i < scalar(@transcripts); $i++) {
			my @coordinates_array = @{$self->{'coord_hash'}->{$chromosome}->{$transcripts[$i]}};	
			for (my $j = 0; $j < scalar(@coordinates_array); $j++) {
				eval {	
					my $score = $rp_detector->average_regulatory_potential_score($chromosome, $coordinates_array[$j]->{'start'}, $coordinates_array[$j]->{'end'});
					$score_hash{$chromosome}{$transcripts[$i]}{$j} = $score;
				};
				if ($@) {
					print STDERR $@;	
				}
			
			}
		}
	}
	return \%score_hash;
}


######################################## SPECIAL, DOES NOT RETURN SCORE, RETURNS AN INFORMATION STRING

sub calculate_repeat_content_information {
	my $self = shift;
	my $rcf_detector = new Chum::Utils::RepeatContentFinder();
	my %score_hash;
	foreach my $chromosome (keys %{$self->{'coord_hash'}}) {
		my @transcripts = keys %{$self->{'coord_hash'}->{$chromosome}};
		for (my $i = 0; $i < scalar(@transcripts); $i++) {
			my @coordinates_array = @{$self->{'coord_hash'}->{$chromosome}->{$transcripts[$i]}};	
			for (my $j = 0; $j < scalar(@coordinates_array); $j++) {
				eval {	
					my $repeat_object = $rcf_detector->_get_repeat_content_for_chr_start_end($chromosome, $coordinates_array[$j]->{'start'}, $coordinates_array[$j]->{'end'}, "1", "100");
					my $repeat_line = "##REPEATS:";
					my $repeats_hashref =  $repeat_object->{'repeats'};
					for my $repeat_name (keys %$repeats_hashref) {
						$repeat_line .= $repeat_name . ":" . $repeats_hashref->{$repeat_name} . ",";
					}
					$repeat_line .= "##INREPEAT:" . $repeat_object->{'in_repeat'};
					$repeat_line .= "##SHORTEVENTS:" . $repeat_object->{'events_short'};
					$repeat_line .= "##LONGEVENTS:" . $repeat_object->{'events_long'};
					$repeat_line .= "##LENGTH:" . $repeat_object->{'length'} . "##";
					$score_hash{$chromosome}{$transcripts[$i]}{$j} = $repeat_line;
				};
				if ($@) {
					print STDERR $@;	
				}
			
			}
		}
	}
	return \%score_hash;	
}

1;