#
# Chum::VariantSequence
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::VariantSequence - A variant sequence

=head1 SYNOPSIS

=head1 DESCRIPTION

A sequence with an assigned allele inserted.  Possess sequence flanking the allele.
The position of the allele within the sequence.  And the allele sequence itself.
All sequences are Bio::Seq objects

Example:
my $variant_sequence = new Chum::VariantSequence( 	'id' => "Homo_sapiens",
							'sequence' => new Bio::Seq(-id => "sequence", -seq => "CCCCCTGGGGG"),
							'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => "CCCCC"),
							'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => "GGGGG"),
							'allele_sequence' => new Bio::Seq(-id => "allele", -seq => "T"));
$variant_sequence->start; #Prints 5
$variant_sequence->end; #Prints 5


=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::VariantSequence;

use vars qw(@ISA @VARIANTSEQUENCE_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @VARIANTSEQUENCE_PARAMS = qw(id sequence allele_left_flank_sequence allele_right_flank_sequence allele_sequence _start _end);
        foreach my $attr (@VARIANTSEQUENCE_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		if (!defined($self->{'id'})) { $self->throw("No id defined for Chum::VariantSequence");}
		if (!defined($self->{'sequence'})) { $self->throw("No sequence defined for Chum::VariantSequence");}
		unless ($self->{'sequence'}->isa("Bio::Seq") || $self->{'sequence'}->isa("Bio::LocatableSeq")) { $self->throw("sequence must be a Bio::Seq object"); }
		if (!defined($self->{'allele_left_flank_sequence'})) { $self->throw("No allele_left_flank_sequence defined for Chum::VariantSequence");}
		unless ($self->{'allele_left_flank_sequence'}->isa("Bio::Seq") || $self->{'allele_left_flank_sequence'}->isa("Bio::LocatableSeq")) { $self->throw("allele_left_flank_sequence must be a Bio::Seq object"); }
		if (!defined($self->{'allele_right_flank_sequence'})) { $self->throw("No allele_right_flank_sequence defined for Chum::VariantSequence");}
		unless ($self->{'allele_right_flank_sequence'}->isa("Bio::Seq") || $self->{'allele_right_flank_sequence'}->isa("Bio::LocatableSeq")) { $self->throw("allele_right_flank_sequence must be a Bio::Seq object"); }
		if (!defined($self->{'allele_sequence'})) { $self->throw("No allele_sequence defined for Chum::VariantSequence");}
		unless ($self->{'allele_sequence'}->isa("Bio::Seq") || $self->{'allele_sequence'}->isa("Bio::LocatableSeq")) { $self->throw("allele_sequence must be a Bio::Seq object"); }
		
		$self->_load_coordinates();
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _load_coordinates {
	my $self = shift;
	my $start = index(lc($self->sequence->seq), lc($self->allele_sequence->seq) . lc($self->allele_right_flank_sequence->seq)) + 1;
	if ($start < 0) { die "Error: sequence does not contain allele sequence and flank: cannot calculate allele position"};
	my $end = $start + length($self->allele_sequence->seq) - 1;
	if (length($self->allele_sequence->seq) == 0) { $end = $end + 1; } ##Special case for indels
	$self->{'_start'} = $start;
	$self->{'_end'} = $end;
}

=head2 id

 Usage   : $variant_sequence->id;
		a display name for this sequence
 Returns : string
 Args    : none

=cut

sub id {
	my $self = shift;
	return $self->{'id'};
}

=head2 sequence

 Usage   : $variant_sequence->sequence;
		a sequence with the allele inserted into it
 Returns : Bio::Seq
 Args    : none

=cut

sub sequence {
	my $self = shift;
	return $self->{'sequence'};
}

=head2 allele_sequence

 Usage   : $variant_sequence->allele_sequence;
		the allele sequence
 Returns : Bio::Seq
 Args    : none

=cut

sub allele_sequence {
	my $self = shift;
	return $self->{'allele_sequence'};
}

=head2 allele_left_flank_sequence

 Usage   : $variant_sequence->allele_left_flank_sequence;
		a 5 prime sequence flanking the allele
 Returns : Bio::Seq
 Args    : none

=cut

sub allele_left_flank_sequence {
	my $self = shift;
	return $self->{'allele_left_flank_sequence'};
}

=head2 allele_right_flank_sequence

 Usage   : $variant_sequence->allele_right_flank_sequence;
		a 3 prime sequence flanking the allele
 Returns : Bio::Seq
 Args    : none

=cut

sub allele_right_flank_sequence {
	my $self = shift;
    return $self->{'allele_right_flank_sequence'};
}

=head2 start

 Usage   : $variant_sequence->start;
		the start position of the allele in the sequence
 Returns : integer
 Args    : none

=cut

sub start {
	my $self = shift;
    return $self->{'_start'};
}

=head2 end

 Usage   : $variant_sequence->end;
		the end position of the allele in the sequence
 Returns : integer
 Args    : none

=cut

sub end {
	my $self = shift;
    return $self->{'_end'};
}

=head2 get_sequence_from_flank

 Usage   : $variant_sequence->get_sequence_from_flank;
	get a flank sequence around object, must be smaller flank than half the length of the sequence
 Returns : new Bio::Seq object
 Args    : flank, integer

=cut

sub get_sequence_from_flank {
	my $self = shift;
	my $flank = shift;

	my $right_index = index(lc($self->sequence->seq), lc($self->allele_right_flank_sequence->seq));
	my $left_index = index(lc($self->sequence->seq), lc($self->allele_left_flank_sequence->seq));

	my $right_substr = substr(lc($self->sequence->seq), $right_index, $flank);
	my $left_substr = substr(lc($self->sequence->seq), $left_index + length($self->allele_left_flank_sequence->seq) - $flank, $flank);

	return new Bio::Seq(	-id => "flank:$flank",
				-seq => lc($left_substr . $self->allele_sequence->seq . $right_substr)); 
}

=head2 is_sequence_feature_overlapping

 Usage   : $variant_sequence->is_sequence_feature_overlapping($sequence_feature);
	determine if a Bio::SeqFeature object is overlapping the allele in this Chum::VariantSequence object
 Returns : boolean
 Args    : Bio::SeqFeature

=cut

sub is_sequence_feature_overlapping {
	my $self = shift;
	my $sequence_feature = shift;
	my $start = $sequence_feature->start <= $sequence_feature->end ? $sequence_feature->start : $sequence_feature->end;
	my $end = $sequence_feature->start <= $sequence_feature->end ? $sequence_feature->end : $sequence_feature->start;
	return ($start <= $self->end && $end >= $self->start);
}

=head2 is_start_stop_overlapping

 Usage   : $variant_sequence->is_start_stop_overlapping(10, 20);
	determine if an object's start and end is overlapping the allele in this Chum::VariantSequence object
 Returns : boolean
 Args    : start, integer
	end, integer

=cut

sub is_start_stop_overlapping {
	my $self = shift;
	my $start = shift;
	my $end = shift;
	#if ($start <= $self->end && $end >= $self->start) {
	#	print STDERR $start . " " . $end . " vs " . $self->start . " " . $self->end . "\n";
	#}
	return ($start <= $self->end && $end >= $self->start);
}

1;
