#!/usr/bin/perl
use strict;

use Chum::Statistics::AnalysisResult;
use Chum::Utils::ExpectancyManager;
use Data::Dumper;

############################################################CONSTANTS
##
my $STARSON = 1;
my $SPECCHAR = "NA";
my $NORMALIZE_ALL_EXPECTANCIES = 1;
my $EXPECTANCY_LOADER = undef;
if ($NORMALIZE_ALL_EXPECTANCIES) { $EXPECTANCY_LOADER = new Chum::Utils::ExpectancyManager(); }

############################################################SETUP PROGRAM
##
my $groups_file;
my $raw_results_directory;
my $transcript_chromosome_map;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-d/) {
        $raw_results_directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-t/) {
        $transcript_chromosome_map = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($groups_file)) { die("No input group file defined"); }
unless (-e $groups_file) { die("Input group file does not exist"); }
if (!defined($raw_results_directory)) { die("No raw results directory defined"); }
unless (-d $raw_results_directory) { die("Raw results directory does not exist"); }
if (!defined($transcript_chromosome_map)) { die("No transcript chromosome map file defined"); }
unless (-e $transcript_chromosome_map) { die("Transcript chromosome map file does not exist"); }

############################################################LOAD CHROMOSOME MAP
##
my %TRANSCRIPT_CHROMOSOME_MAP;
open(TCM, $transcript_chromosome_map) || die ("Could not open transcript chr map: " . $transcript_chromosome_map);
while (my $line = <TCM>) {
    chomp($line);
    if ($line =~ /([\S]+)[\s]+([\S]+)/g) {
	$TRANSCRIPT_CHROMOSOME_MAP{$1} = $2;
    }
}
close(TCM);

############################################################OPEN GROUP FILE
##
my %GROUPS;
my %TRANSCRIPTS;
open(GF, $groups_file) || die ("Could not open group file: " . $groups_file);
while (my $line = <GF>) {
    chomp($line);
    if ($line =~ /([\S]+):([\S\s]+)/g) {
	my $transcript_stable_id = $1;
        my $split_line = $2;
        $TRANSCRIPTS{$transcript_stable_id} = 1;
        my @tokens = split /[\s]+/, $split_line;
        foreach my $token (@tokens) {
            if ($token =~ /[\S]+/g) {    
                $GROUPS{$token} = $transcript_stable_id;
            }
        }
    }
}
close(GF);

############################################################RUN TESTS
##
my $resultshashref = loadFiles(\%GROUPS, $raw_results_directory);
my %results = %$resultshashref;

############################################################LOAD AVERAGES FOR STATISTICAL ANALYSIS
##
my %averages;
for my $gene (keys %results) {
    print STDERR "Loading averages for $gene\n";
    $averages{$gene} = &loadAverages($results{$gene});  
}

my $index = 1;
my %tests = (	"transfac.difference" => $index++,
		"weeder.difference" => $index++,
		"weeder.maximum" => $index++,
		"motifsampler.difference" => $index++,
		"motifsampler.maximum" => $index++,
		"bend.difference" => $index++,
		"curve.difference" => $index++,
		"gc.difference" => $index++,
		"temperature.difference" => $index++,
                "transition.transversion" => $index++,
		"distance" => $index++,
		"ln.distance" => $index++,
		"repeat.density" => $index++,
		"in.repeat" => $index++,
		"repeat.short.events" => $index++,
		"repeat.long.events" => $index++,
		"maf" => $index++,
		"daf" => $index++,
		"coexpression.jaspar.difference" => $index++,
		"in.cpg" => $index++,
		###"nhgri.dnase" => $index++,
		"noble.dnase" => $index++,
		"phastcons" => $index++,
		"rp" => $index++,
		"alignment.distance" => $index++);

############################################################OUTPUT DATA
##

##HEADER DATA
print "id" . "\t" . "transcript" . "\t" . "case";
foreach my $test (sort { $tests{$a} <=> $tests{$b} } keys %tests) {
	print "\t" . $test;
}
print "\n";

##DATA DATA
for my $gene (keys %results) {
	my $results_hashref = $results{$gene};
	my %analysis_results_hash = %$results_hashref;
	for my $id (keys %analysis_results_hash) {
		my $result = $analysis_results_hash{$id};
                
                if ($result->isNotSNP()) {
                    print STDERR "Skipping " . $result->{'id'} . "\n";  ##ENSURE THAT WE SKIP NON-SNPS
                    next;  ##PROBABLY NOT A SNP
                }
               
                my $transcript_stable_id = $GROUPS{$result->{'id'}}; 
                my $closest_gene = $result->{'distance'}->{$transcript_stable_id}->{'gene'};
                my $closest_distance = $result->{'distance'}->{$transcript_stable_id}->{'distance'};
                my $closest_ln_distance = $result->{'distance'}->{$transcript_stable_id}->{'ln_distance'};
    
                if (abs($closest_distance) > 2001) {
                    warn($result->{'id'} . " is outside range for " . $transcript_stable_id . " with: " . $closest_distance);
                    next; ##NOT WITHIN SEARCH REGION WE ARE INTERESTED IN
                } elsif (!defined($closest_gene)) {
                    warn("Could not find closest gene for: " . $transcript_stable_id . " in " . $result->{'id'});
                    next;
                }
                my $OUTPUT_LINE = "";
                $OUTPUT_LINE .= $result->{'id'} . "\t" . $GROUPS{$result->{'id'}};
                
                if ($result->{'id'} =~ /OREG[0-9]+/) {
                    $OUTPUT_LINE .= "\tPOSITIVE\t";
                } elsif ($result->{'id'} =~ /rs[0-9]+/) {
                    $OUTPUT_LINE .= "\tNEUTRAL\t";
                } else {
                    die("Unknown id: " . $result->{'id'});   
                }
                
                if (defined($tests{'transfac.difference'})) {
                    if ($result->{'transfac'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'transfac'}->{'score'} - $averages{$gene}{'transfac'};
                        $OUTPUT_LINE .=  $result->{'transfac'}->{'score'} . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'weeder.difference'})) {
                    if ($result->{'weeder'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'weeder'}->{'score'} - $averages{$gene}{'weeder-diff'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'weeder.maximum'})) {
                    if ($result->{'weeder'}->{'max_score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'weeder'}->{'max_score'} - $averages{$gene}{'weeder-max'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'motifsampler.difference'})) {
                    if ($result->{'motifsampler'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'motifsampler'}->{'score'} - $averages{$gene}{'motifsampler-diff'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'motifsampler.maximum'})) {
                    if ($result->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'motifsampler'}->{'max_score'} - $averages{$gene}{'motifsampler-max'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'bend.difference'})) {
                    if ($result->{'bend'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'bend'}->{'score'} - $averages{$gene}{'bend'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'curve.difference'})) {
                    if ($result->{'curve'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'curve'}->{'score'} - $averages{$gene}{'curve'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'gc.difference'})) {
                    if ($result->{'gc'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'gc'}->{'score'} - $averages{$gene}{'gc'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'temperature.difference'})) {
                    if ($result->{'temperature'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'temperature'}->{'score'} - $averages{$gene}{'temperature'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                
                if (defined($tests{'transition.transversion'})) {
                    if ($result->{'transition-transversion'} ne "UNDEFINED" && $result->{'transition-transversion'} ne "") {
                        $OUTPUT_LINE .=  $result->{'transition-transversion'} . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                            
                ###DISTANCE
                if (defined($tests{'distance'})) {
                    if (defined($closest_distance)) {
                        my $divergence_from_average = $closest_distance - $averages{$gene}{'distance'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'ln.distance'})) {
                    if (defined($closest_ln_distance)) {
                        my $divergence_from_average = $closest_ln_distance - $averages{$gene}{'ln-distance'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                
                my $repeat_density = 0;
                if ($result->{'repeat'} != 0) {
                        $repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
                }
                my $CHROMOSOME = $TRANSCRIPT_CHROMOSOME_MAP{$transcript_stable_id};
                my $expectancy_adjustment_in_repeat_value = 0;
                my $expectancy_adjustment_short_event = 0;
                my $expectancy_adjustment_long_event = 0;
                if ($NORMALIZE_ALL_EXPECTANCIES) {
                    $expectancy_adjustment_in_repeat_value = $EXPECTANCY_LOADER->get_in_repeat_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                    $expectancy_adjustment_short_event = $EXPECTANCY_LOADER->get_short_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                    $expectancy_adjustment_long_event = $EXPECTANCY_LOADER->get_long_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                }
            
                if (defined($tests{'repeat.density'})) {
                    my $divergence_from_average = $repeat_density - $averages{$gene}{'repeat-density'};
		    $OUTPUT_LINE .=  $divergence_from_average . "\t";
                }
                if (defined($tests{'in.repeat'})) {
                    my $divergence_from_average = $result->{'repeat'}->{'in_repeat'} - $averages{$gene}{'in-repeat'} - $expectancy_adjustment_in_repeat_value;
		    $OUTPUT_LINE .=  $divergence_from_average . "\t";
                }
                if (defined($tests{'repeat.short.events'})) {
                    my $divergence_from_average = $result->{'repeat'}->{'short_events'} - $averages{$gene}{'repeat-short-events'} - $expectancy_adjustment_short_event;
		    $OUTPUT_LINE .=  $divergence_from_average . "\t";
                }
                if (defined($tests{'repeat.long.events'})) {
                    my $divergence_from_average = $result->{'repeat'}->{'long_events'} - $averages{$gene}{'repeat-long-events'} - $expectancy_adjustment_long_event;
		    $OUTPUT_LINE .=  $divergence_from_average . "\t";
                }
                
                ###MAF
                my $printed = 0;
                if (defined($tests{'maf'})) {
                    if (defined($result->{'minor_allele_frequency'}) && $result->{'minor_allele_frequency'} != 0) {
                        if (defined($result->{'minor_allele_frequency'}->{'ALL'})) {
                            if (defined($result->{'minor_allele_frequency'}->{'ALL'}->{'frequency'})) {
                                if (defined($tests{'maf'})) {
                                    my $divergence_from_average = $result->{'minor_allele_frequency'}->{'ALL'}->{'frequency'} - $averages{$gene}{'maf'};
                                    $OUTPUT_LINE .=  $divergence_from_average  . "\t";
                                    $printed = 1;
                                }
                            }
                        }
                    }
                }
                unless($printed) {
                    if (defined($tests{'maf'})) {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
            
                if (defined($tests{'daf'})) {
                    if ($result->{'daf'} ne "UNDEFINED" && defined($result->{'daf'})) {
                        if (defined($result->{'daf'}->{'ALL'} )) {
                            my $divergence_from_average = $result->{'daf'}->{'ALL'} - $averages{$gene}{'daf'};
			    $OUTPUT_LINE .=  $divergence_from_average . "\t";
                        } else {
                            $OUTPUT_LINE .=  $SPECCHAR . "\t";
                        }
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                
                
                if (defined($tests{'coexpression.jaspar.difference'})) {
                    if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'coexpression-jaspar'}->{'score'} - $averages{$gene}{'coexpression-jaspar'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                
                my $expectancy_adjustment_in_cpg = 0;
                my $expectancy_adjustment_nhgri = 0;
                my $expectancy_adjustment_noble = 0;
                my $expectancy_adjustment_phastcons = 0;
                my $expectancy_adjustment_regulatory_potential = 0;
                if ($NORMALIZE_ALL_EXPECTANCIES) {
                    $expectancy_adjustment_in_cpg = $EXPECTANCY_LOADER->get_in_cpg_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                    $expectancy_adjustment_nhgri = $EXPECTANCY_LOADER->get_nhgri_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                    $expectancy_adjustment_noble = $EXPECTANCY_LOADER->get_noble_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                    $expectancy_adjustment_phastcons = $EXPECTANCY_LOADER->get_phastcons_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                    $expectancy_adjustment_regulatory_potential = $EXPECTANCY_LOADER->get_regulatory_potential_expectancy_value($CHROMOSOME, (0 - $closest_distance));
                }
                if (defined($tests{'in.cpg'})) {
                    if ($result->{'cpg'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'cpg'}->{'score'} - $averages{$gene}{'in-cpg'} - $expectancy_adjustment_in_cpg;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'nhgri.dnase'})) {
                    if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'nhgri-dnase'}->{'score'} - $averages{$gene}{'nhgri-dnase'} - $expectancy_adjustment_nhgri;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'noble.dnase'})) {
                    if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'noble-dnase'}->{'score'} - $averages{$gene}{'noble-dnase'} - $expectancy_adjustment_noble;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'phastcons'})) {
                    if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'phastcons'}->{'score'} - $averages{$gene}{'phastcons'} - $expectancy_adjustment_phastcons;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'rp'})) {
                    if ($result->{'rp'}->{'score'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'rp'}->{'score'} - $averages{$gene}{'rp'} - $expectancy_adjustment_regulatory_potential;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR . "\t";
                    }
                }
                if (defined($tests{'alignment.distance'})) {
                    if ($result->{'alignment_distance'} ne "UNDEFINED") {
                        my $divergence_from_average = $result->{'alignment_distance'} - $averages{$gene}{'alignment-distance'};
			$OUTPUT_LINE .=  $divergence_from_average;
                    } else {
                        $OUTPUT_LINE .=  $SPECCHAR;
                    }
                }
                
                chomp($OUTPUT_LINE);
                $OUTPUT_LINE .=  "\n";
                print $OUTPUT_LINE;
        }
}
exit(0);


############################################################PRIVATE SUBROUTINES
##
sub loadFiles {
	my $groupshashref = shift;
        my $raw_results_directory = shift;
        
	my %results = ();
	foreach my $id (keys %$groupshashref) {
		my $result = new Chum::Statistics::AnalysisResult( 'file' => $raw_results_directory . $id . ".results");
                $results{$groupshashref->{$id}}{$id} = $result;
	}
	return \%results;
}

sub loadAverages() {
	my $results_hashref = shift;
	my %results_hash = %$results_hashref;
	
	#######################################################################SETUP TOTALS AND COUNTS HASHES
	my %totals = ("transfac" => 0, "weeder-diff" => 0, "weeder-max" => 0, "motifsampler-diff" => 0, "motifsampler-max" => 0,
			"bend" => 0, "curve" => 0, "gc" => 0, "temperature" => 0, "distance" => 0, "ln-distance" => 0,
			"repeat-density" => 0, "in-repeat" => 0,
			"repeat-short-events" => 0, "repeat-long-events" => 0, "maf-rare" => 0, "maf-common" => 0, "maf-unknown" => 0,
			"coexpression-jaspar" => 0, "in-cpg" => 0, "nhgri-dnase" => 0, "noble-dnase" => 0, "phastcons" => 0,
			"rp" => 0, "maf" => 0, "daf" => 0, "alignment-distance" => 0);
	my %counts = (	"transfac" => 0, "weeder-diff" => 0, "weeder-max" => 0, "motifsampler-diff" => 0, "motifsampler-max" => 0,
			"bend" => 0, "curve" => 0, "gc" => 0, "temperature" => 0, "distance" => 0, "ln-distance" => 0,
			"repeat-density" => 0, "in-repeat" => 0,
			"repeat-short-events" => 0, "repeat-long-events" => 0, "maf-rare" => 0, "maf-common" => 0, "maf-unknown" => 0,
			"coexpression-jaspar" => 0, "in-cpg" => 0, "nhgri-dnase" => 0, "noble-dnase" => 0, "phastcons" => 0,
			"rp" => 0, "maf" => 0, "daf" => 0, "alignment-distance" => 0);
	
	
	#######################################################################PROCESS ALL RESULT FILES
	for my $id (keys %results_hash) {
		my $result = $results_hash{$id};

		if ($result->isNotSNP()) {
                    next;   
                }

		my $transcript_stable_id = $GROUPS{$result->{'id'}}; 
		my $closest_gene = $result->{'distance'}->{$transcript_stable_id}->{'gene'};
		my $closest_distance = $result->{'distance'}->{$transcript_stable_id}->{'distance'};
		my $closest_ln_distance = $result->{'distance'}->{$transcript_stable_id}->{'ln_distance'};
    
		if (abs($closest_distance) > 6001) {
		    warn($result->{'id'} . " is outside range for " . $transcript_stable_id . " with: " . $closest_distance);
		    next; ##NOT WITHIN SEARCH REGION WE ARE INTERESTED IN
		} elsif (!defined($closest_gene)) {
		    warn("Could not find closest gene for: " . $transcript_stable_id . " in " . $result->{'id'});
		    next;
		}	

		################################################################
		######################################@TODO PROCESS ONLY SNPS
		################################################################

		##############################################ADD ANALYSIS VALUES TO AVERAGE HASH
		if ($result->{'transfac'}->{'score'} ne "UNDEFINED") {
		    $totals{'transfac'} += $result->{'transfac'}->{'score'};
		    $counts{'transfac'} += 1;
		}
		if ($result->{'weeder'}->{'score'} ne "UNDEFINED") {
		    $totals{'weeder-diff'} += $result->{'weeder'}->{'score'};
		    $counts{'weeder-diff'} += 1;
		}
		if ($result->{'weeder'}->{'max_score'} ne "UNDEFINED") {
		    $totals{'weeder-max'} += $result->{'weeder'}->{'max_score'};
		    $counts{'weeder-max'} += 1;
		}
		if ($result->{'motifsampler'}->{'score'} ne "UNDEFINED") {
		    $totals{'motifsampler-diff'} += $result->{'motifsampler'}->{'score'};
		    $counts{'motifsampler-diff'} += 1;
		}
		if ($result->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
		    $totals{'motifsampler-max'} += $result->{'motifsampler'}->{'max_score'};
		    $counts{'motifsampler-max'} += 1;
		}
		if ($result->{'bend'}->{'score'} ne "UNDEFINED") {
		    $totals{'bend'} += $result->{'bend'}->{'score'};
		    $counts{'bend'} += 1;
		}
		if ($result->{'curve'}->{'score'} ne "UNDEFINED") {
		    $totals{'curve'} += $result->{'curve'}->{'score'};
		    $counts{'curve'} += 1;
		}
		if ($result->{'gc'}->{'score'} ne "UNDEFINED") {
		    $totals{'gc'} += $result->{'gc'}->{'score'};
		    $counts{'gc'} += 1;
		}
		if ($result->{'temperature'}->{'score'} ne "UNDEFINED") {
		    $totals{'temperature'} += $result->{'temperature'}->{'score'};
		    $counts{'temperature'} += 1;
		}
		if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") {
		    $totals{'coexpression-jaspar'} += $result->{'coexpression-jaspar'}->{'score'};
		    $counts{'coexpression-jaspar'} += 1;
		}

		##############################################GET CLOSEST GENE INFORMATION
		$totals{'distance'} += $closest_distance;
		$counts{'distance'} += 1;
		$totals{'ln-distance'} += $closest_ln_distance;
		$counts{'ln-distance'} += 1;
		
		#my $CHROMOSOME = getChromosomeForGene($closest_gene);
		my $CHROMOSOME = $TRANSCRIPT_CHROMOSOME_MAP{$transcript_stable_id};

		##############################################GET REPEAT INFORMATION
		my $repeat_density = 0;
		if ($result->{'repeat'} != 0) {
			$repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
		}
		$totals{'repeat-density'} += $repeat_density;
		$counts{'repeat-density'} += 1;
		
		my $expectancy_adjustment_in_repeat_value = 0;
		my $expectancy_adjustment_short_event = 0;
		my $expectancy_adjustment_long_event = 0;
		if ($NORMALIZE_ALL_EXPECTANCIES) {
		    $expectancy_adjustment_in_repeat_value = $EXPECTANCY_LOADER->get_in_repeat_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_short_event = $EXPECTANCY_LOADER->get_short_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_long_event = $EXPECTANCY_LOADER->get_long_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    
		}
		$totals{'in-repeat'} += $result->{'repeat'}->{'in_repeat'} - $expectancy_adjustment_in_repeat_value;
		$counts{'in-repeat'} += 1;
		$totals{'repeat-short-events'} += $result->{'repeat'}->{'short_events'} - $expectancy_adjustment_short_event;
		$counts{'repeat-short-events'} += 1;
		$totals{'repeat-long-events'} += $result->{'repeat'}->{'long_events'} - $expectancy_adjustment_long_event;
		$counts{'repeat-long-events'} += 1;

		
		if (defined($result->{'minor_allele_frequency'}) && $result->{'minor_allele_frequency'} != 0) {
		    if (defined($result->{'minor_allele_frequency'}->{'ALL'})) {
			if (defined($result->{'minor_allele_frequency'}->{'ALL'}->{'frequency'})) {
			    $totals{'maf'} += $result->{'minor_allele_frequency'}->{'ALL'}->{'frequency'};
			    $counts{'maf'} += 1;
			}
		    }
		}
		
		if ($result->{'daf'} ne "UNDEFINED") {
		    if (defined($result->{'daf'}->{'ALL'})) {
			$totals{'daf'} += $result->{'daf'}->{'ALL'};
			$counts{'daf'} += 1;
		    }
		}
		
		##############################################GET EXPECTANCY ADJUSTED VALUES FOR ANNOTATION FEATURES
		my $expectancy_adjustment_in_cpg = 0;
		my $expectancy_adjustment_nhgri = 0;
		my $expectancy_adjustment_noble = 0;
		my $expectancy_adjustment_phastcons = 0;
		my $expectancy_adjustment_regulatory_potential = 0;
		if ($NORMALIZE_ALL_EXPECTANCIES) {
		    $expectancy_adjustment_in_cpg = $EXPECTANCY_LOADER->get_in_cpg_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_nhgri = $EXPECTANCY_LOADER->get_nhgri_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_noble = $EXPECTANCY_LOADER->get_noble_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_phastcons = $EXPECTANCY_LOADER->get_phastcons_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_regulatory_potential = $EXPECTANCY_LOADER->get_regulatory_potential_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		}
		if ($result->{'in-cpg'}->{'score'} ne "UNDEFINED") {
		    $totals{'in-cpg'} += $result->{'cpg'}->{'score'} - $expectancy_adjustment_in_cpg;
		    $counts{'in-cpg'} += 1;
		}
		if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") {
		    $totals{'nhgri-dnase'} += $result->{'nhgri-dnase'}->{'score'} - $expectancy_adjustment_nhgri;
		    $counts{'nhgri-dnase'} += 1;
    		}
		if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") {
		    $totals{'noble-dnase'} += $result->{'noble-dnase'}->{'score'} - $expectancy_adjustment_noble;
		    $counts{'noble-dnase'} += 1;
		}
		if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") {
		    $totals{'phastcons'} += $result->{'phastcons'}->{'score'} - $expectancy_adjustment_phastcons;
		    $counts{'phastcons'} += 1;
		}
		if ($result->{'rp'}->{'score'} ne "UNDEFINED") {
		    $totals{'rp'} += $result->{'rp'}->{'score'} - $expectancy_adjustment_regulatory_potential;
		    $counts{'rp'} += 1;
		}
		if ($result->{'alignment_distance'} ne "UNDEFINED") {
		    $totals{'alignment-distance'} += $result->{'alignment_distance'};
		    $counts{'alignment-distance'} += 1;
		}
	}
	for my $key (keys %totals) {
		eval {
		    $totals{$key} = $totals{$key} / $counts{$key};
		}; if (@$) {
                    print STDERR "Setting $key to 0\n";
		    $totals{$key} = 0;
		}
	}
        #print Dumper %totals;
	return \%totals;
}
