#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Utils::AlleleFrequencyFinder;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 100;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $results = "";
my $aff = new Chum::Utils::AlleleFrequencyFinder();
my $genotype_populations_arrayref = $aff->get_allele_frequencies_by_genotype_population($stack_object);
my $totals_genotypes_hashref = $aff->get_total_allele_frequency_from_genotype_populations($genotype_populations_arrayref);

##HAVE GENOTYPE INFORMATION
if (defined($genotype_populations_arrayref) && defined($totals_genotypes_hashref)) {
    my @variant_sequences = @{$stack_object->variant_sequences};
    
    ##1. determine alleles and if they should be reverse-complemented
    my @db_alleles = ();
    my $rev_com_required = 1;
    foreach my $allele (@{$totals_genotypes_hashref->{'alleles'}}) {
	if ($allele->{'allele'} =~ /^[ACTGactg\-]+$/g) {
	    push @db_alleles, $allele;
	}
    }
    my @ref_alleles = ( lc($stack_object->reference_sequence->allele_sequence->seq),
			lc($variant_sequences[0]->allele_sequence->seq));
    my @rev_ref_alleles = ( lc($stack_object->reference_sequence->allele_sequence->revcom->seq),
			    lc($variant_sequences[0]->allele_sequence->revcom->seq));
    my $ref_match = 0;
    my $rev_match = 0;
    foreach my $ref_allele (@ref_alleles) {
	foreach my $db_allele (@db_alleles) {
	    if ($ref_allele eq lc($db_allele->{'allele'})) {
		$ref_match++;
	    }
	}
    }
    foreach my $rev_ref_allele (@rev_ref_alleles) {
	foreach my $db_allele (@db_alleles) {
	    if ($rev_ref_allele eq lc($db_allele->{'allele'})) {
		$rev_match++;
	    }
	}
    }
    if ($rev_match > $ref_match) {  ##More of the alleles from the reference/variant seq are matched by rev-com of the obtained db_alleles
	$rev_com_required = 0;
    }
    
    ##2. print out total allele frequency and alleles
    foreach my $allele (@db_alleles) {
	my $allele_seq;
	if ($rev_com_required) {
	    my $allele_seqobj = new Bio::Seq(   -seq => lc($allele->{'allele'}),
                                                        -id => "tmp");
            $allele_seq = lc($allele_seqobj->revcom->seq);
	} else {
	    $allele_seq = $allele->{'allele'};
	}
	$results .= "ALL:" . $totals_genotypes_hashref->{'samples'} . ":" . uc($allele_seq) . ":" . $allele->{'frequency'} . "\n";
    }
    
    ##3. print out all population alleles and frequencies
    foreach my $population (@$genotype_populations_arrayref) {
	foreach my $allele (@{$population->{'alleles'}}) {
	    my $allele_seq;
	    if ($rev_com_required) {
	        my $allele_seqobj = new Bio::Seq(   -seq => lc($allele->{'allele'}),
                                                    -id => "tmp");
		$allele_seq = lc($allele_seqobj->revcom->seq);
	    } else {
		$allele_seq = $allele->{'allele'};
	    }
	    $results .=  $population->{'population_location_id'} .  ":" . $population->{'samples'} . ":" .  uc($allele_seq) . ":" . $allele->{'frequency'} . "\n";
	}
    }
}

#OLD METHOD, DID NOT PRINT OUT BASED ON POPULATIONS!
#
#if (defined($totals_genotypes_hashref)) {
#    my $printed = 0;
#    my @variant_sequences = @{$stack_object->variant_sequences};
#    if (scalar(@variant_sequences) > 0) {
#        foreach my $allele (@{$totals_genotypes_hashref->{'alleles'}}) {
#            print STDERR "ALLELE: " . $allele->{'allele'} . "\n";
#            if (lc($stack_object->reference_sequence->allele_sequence->seq) eq lc($allele->{'allele'}) ||
#                lc($variant_sequences[0]->allele_sequence->seq) eq lc($allele->{'allele'})) {
#                $results .= $allele->{'allele'} . ":" . $allele->{'frequency'} . "\n";
#                $printed = 1;
#            }
#        }
#        if (!$printed) {
#            foreach my $allele (@{$totals_genotypes_hashref->{'alleles'}}) {
#                if ($allele->{'allele'} =~ /^[ACTGactg\-]+$/g) {
#                    my $allele_seqobj = new Bio::Seq(   -seq => lc($allele->{'allele'}),
#                                                        -id => "tmp");
#                    my $reverse_allele = $allele_seqobj->revcom->seq;
#                    if (lc($stack_object->reference_sequence->allele_sequence->seq) eq lc($reverse_allele) ||
#                        lc($variant_sequences[0]->allele_sequence->seq) eq lc($reverse_allele)) {
#                        $results .= $allele->{'allele'} . ":" . $allele->{'frequency'} . "\n";
#                    }
#                }
#            }
#        }
#    }
#}

print $results . "\n";


exit(0);
