#!/usr/bin/perl
use strict;
use Bio::AlignIO;
use Bio::EnsEMBL::Registry;
use Bio::Factory::EMBOSS;
use Bio::Seq;
use Data::Dumper;
use SOAP::Lite;

##############################################TEST
&getAllelesUsingNeedle(lc("ttgtgtcttgccatgctaaaggacgtcacattgca"),lc("ttgtgtcttgcGatgctaaaggacgtcacattgca"));
exit(0);

##############################################LOAD THE INPUT PARAMETERS
my $filename;
my $id;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $filename = uc($ARGV[++$i]);
    } elsif ($ARGV[$i] =~ /-i/) {
        $id = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($filename)) { die("Stack file is undefined\n"); }
unless (-e $filename) { die("Stack file: $filename does not exist\n"); }
if (!defined($id)) { die("Id is not defined\n"); }
unless ($id =~ /OREG[0-9]{7}/ || $id =~ /rs[0-9]+/) {die("Id does not match ORegAnno or dbSNP standard ids"); }

##############################################LOAD SEQUENCES
my %SEQUENCE_HASH;
my $in  = Bio::SeqIO->new(  -file => $filename,
                            -format => 'Fasta');
while ( my $seq = $in->next_seq() ) {
    $SEQUENCE_HASH{lc($seq->display_id())} = $seq->seq;
}

##############################################THE VARIANT INFO TO LOAD
my $leftseq;
my $rightseq;
my @alleleseqs;

##############################################GET THE VARIANT INFORMATION
if ($id =~ /rs[0-9]+/) {
    my $alleleseqsref;
    ($leftseq, $rightseq, $alleleseqsref) = &loadFromDbSnp($id);
    @alleleseqs = @$alleleseqsref;
} elsif ($id =~ /OREG[0-9]{7}/) {
    my $alleleseqsref;
    ($leftseq, $rightseq, $alleleseqsref) = &loadFromORegAnno($id);
    @alleleseqs = @$alleleseqsref;
} else {
    die("Id is not mapped\n");   
}

print "LEFTSEQ: $leftseq\n";
print "RIGHTSEQ: $rightseq\n";
print Dumper @alleleseqs;

#TODO, Assemble new stack and output flanks around variants

##############################################SUBROUTINES, DBSNP
sub loadFromDbSnp() {
    my $id = shift;
    my ($leftseq, $rightseq);
    my @alleleseqs = ();
    my $reg = "Bio::EnsEMBL::Registry";
    $reg->load_all();
    my $va = $reg->get_adaptor("human", "variation", "Variation");
    my $variation = $va->fetch_by_name($id, 'dbSNP');
    my @alleles = @{$variation->get_all_Alleles()};
    my $vfa = $reg->get_adaptor("human", "variation", "VariationFeature");
    my @variation_features = @{$vfa->fetch_all_by_Variation($variation)};
    if (scalar(@variation_features) != 1) {
        print STDERR "ERROR: Did not find a unique variant for $id\n";
        $reg->disconnect_all();
        die(0);
    }
    my $variation_feature = $variation_features[0];
    my $seq_region_name = $variation_feature->seq_region_name;
    my $start = $variation_feature->start;
    my $end = $variation_feature->end;
    my $strand = $variation_feature->strand;
    my $sa = $reg->get_adaptor("human", "core", "Slice");
    my $leftslice = $sa->fetch_by_region('chromosome', $seq_region_name,  $start - 41, $start - 1, 1);
    my $rightslice = $sa->fetch_by_region('chromosome', $seq_region_name,  $end + 1, $end + 41, 1);
    $leftseq = $leftslice->seq;
    $rightseq = $rightslice->seq;
    if ($strand eq "1") {
        foreach my $allele (@alleles) {
            push @alleleseqs, $allele->allele;
        }
    } elsif ($strand eq "-1") {
        foreach my $allele (@alleles) {
            my $allele_sobj = new Bio::Seq(-id => 'tmp', -seq => $allele->allele);
            push @alleleseqs, $allele_sobj->revcom->seq;
        }
    } else {
        print STDERR "ERROR: Undefined strand for $id\n";
        $reg->disconnect_all();
        die(0);
    }
    $reg->disconnect_all();
    return $leftseq, $rightseq, \@alleleseqs;
}

##############################################SUBROUTINES, OREGANNO
sub loadFromORegAnno() {
    my $id = shift;
    my ($leftseq, $rightseq);
    my @alleleseqs = ();
    my $osa = SOAP::Lite
        -> uri('OregannoServerImpl')
        -> proxy('http://www.bcgsc.ca:8080/oregano/soap/', timeout=> 1000);
    my $response = $osa->searchRecords(
            SOAP::Data->name(field => SOAP::Data->type(string => "stable_id")),
            SOAP::Data->name(query => SOAP::Data->type(string => "$id")));
    if ($response->fault) {
        die($response->faultstring);
    } else {
        my @records = @{$response->result};
        if (scalar(@records) != 1) {
            print STDERR "ERROR: Cannot find unique id for $id\n";
            die(0);
        }
        my $record = $records[0];
        print Dumper $record;
        my @variations = @{$record->{'record'}->{'variationSet'}->{'variation'}};
        if (scalar(@variations) != 1) {
            print STDERR "ERROR: Cannot find variation for $id\n";
            die(0);   
        }
        my $variation = $variations[0];
        ##Load from dbSNP, if available
        if (defined($variation->{'regulatoryVariationCrossReferenceSet'})) {
            my @rvxrefs = @{$variation->{'regulatoryVariationCrossReferenceSet'}->{'regulatoryVariationCrossReference'}};
            if (scalar(@rvxrefs) == 1) {
                my $rvxref = $rvxrefs[0];
                if ($rvxref->{'source'} eq 'dbSNP') {
                    if (defined($rvxref->{'snpId'})) {
                        return &loadFromDbSnp($rvxref->{'snpId'});
                    }
                }
            }
        }
        my $reference_seq = $variation->{'referenceSequence'};
        my $variant_seq = $variation->{'variantSequence'};
        if ((index(lc($SEQUENCE_HASH{'hosa'}), lc($reference_seq)) > 0 && ##Reference sequence is canonical sequence
            index(lc($SEQUENCE_HASH{'hosa'}), lc($variant_seq)) < 0) ) {
            ##determine alleles
            &getAllelesUsingNeedle($reference_seq, $variant_seq);   
           
            
        } elsif (index(lc($SEQUENCE_HASH{'hosa'}), lc($reference_seq)) < 0 && ##Variant sequence is canonical sequence
                 index(lc($SEQUENCE_HASH{'hosa'}), lc($variant_seq)) > 0) {
            ##determine alleles
            &getAllelesUsingNeedle($variant_seq, $reference_seq); 
        }
        
    }
    return $leftseq, $rightseq, \@alleleseqs;
}

sub getAllelesUsingNeedle() {
    my $reference_seq = shift;
    my $variant_seq = shift;
    
    my $ref_seqobj = new Bio::Seq(-id => 'reference', -seq => lc($reference_seq));
    my $var_seqobj = new Bio::Seq(-id => 'variant', -seq => lc($variant_seq));
    
    my $factory = new Bio::Factory::EMBOSS;
    my $tmpfile = "needle." . rand(100) . ".tmp";
    my $needleapp = $factory->program('needle');
    my %input = (   -asequence => $ref_seqobj,
                    -bsequence => $var_seqobj,
                    -outfile  =>  $tmpfile);
    $needleapp->run(\%input);
    my $alignio = Bio::AlignIO->new( -file => $tmpfile,
                                     -format => "emboss");
    while ( my $aln = $alignio->next_aln() ) {
        print Dumper $aln;    
    }
    unlink $tmpfile;
}
